// RELATED SOFTWARE: Scilab >= 5.5.0, Scilab 6.0, Scilab 6.1, Scilab ≥ 2023
// PACKAGE  : uman : Unified user MANual for Scilab, external modules and resources
// VERSION  : 3.2 - 2023-03-26
// AUTHOR   : Samuel GOUGEON - private individual
// COPYRIGHT: Samuel GOUGEON -  2013 - 2023
// LICENSE for uman : CeCILL : http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
//
// DISCLAIMER: This toolbox is provided as is, with NO WARRANTY of any kind.
//             Use it to your own risks & responsability.
//
function uman(varargin)
    tmp = getversion("scilab")
    sciVer = tmp(1) + tmp(2)/10 + tmp(3)/100

    uman_settings(); // Updates global configuration variables according to the preferences file

    RelCharWidth = 1 // init Scaling factor for non latin characters width (ja,zh)
    noWebBrowserMsg = _("uman", "uman: don''t know how to open your web browser\n")

    // FORMATING and partially PARSING OPTIONS
    // ---------------------------------------
    if argn(2)==0 then
        uman("uman","e")
        return
    end
    if argn(2)==3 then
        tmp = convstr(varargin(3))
        if grep(tmp,"l")~=[]
            varargin(1) = varargin(1)+tmp
        else
            varargin(1) = "-" + tmp + part(varargin(1), 2:$)
        end
    end
    if argn(2)==1 then
        if varargin(1)=="-"  then
            fun0 = "-"
            options = ""
        elseif part(varargin(1),1)=="-"
            if grep(varargin(1),"c")~=[] then
                clc
            end
            uman("uman")
            return
        else
            fun0 = varargin(1)
            options = ""
        end
    else
        if varargin(1)=="-" | part(varargin(1),1)~="-" then
            fun0 = varargin(1)
            options = varargin(2)
        else
            fun0 = varargin(2)
            options = varargin(1)
        end
    end

    options = convstr(options)
    options0 = options  // useful in case of redirection (recursive => call)
    [options, LANG] = uman_splitOptionsLang(options)
    // Option "@" is canceled when it is not alone:
    if grep(options,"@")~=[] & length(strsubst(options,"@",""))>0
        //options = strsubst(options, "@", "")
    end

    // ========================================================================
    // MAILING LISTS
    // -------------
    if grep(options, "@")~=[] then
        // Building the query
        if grep(options,"s") <> []
            // StackOverflow : https://stackoverflow.com/help/searching
            // .............
            MailingListsQuery = "https://stackoverflow.com/search?tab=relevance&q=%5Bscilab%5D"
            // In subjects or in full messages?
            fun = fun0
            if fun <> "!" & part(fun,$)=="!"  // search only in subjects of messages.
                s = "+title%3A"
                fun = part(fun,1:$-1)
            else                   // search in subjects + bodies
                s = "+"
            end
            d = "!" // regexp delimiter
    
            // Authors part: NOT (poorly) SUPPORTED
            authors = ""

            // Max age part:
            tmp = tokens(fun, "<")
            if size(tmp,1)>1
                max_age = msprintf("+lastactive%%3A%dd..", evstr(tmp($)));
                fun = strcat(tmp(1:$-1),"<")
            else
                max_age = ""
            end
            fun = stripblanks(fun)
            
            // Topic part
            if fun ~= ""
                tmp = " " + strcat(s + tokens(fun, " "))
            else
                 tmp = ""
            end

        else
            // https://www.mail-archive.com/faq.html#newlist :
            // ............................
            tmp = "https://www.mail-archive.com/search"
            if grep(options,"d") <> []
                MailingListsQuery = tmp + "?l=dev%40lists.scilab.org&q=sort:newest ";
            else
                MailingListsQuery = tmp + "?l=users%40lists.scilab.org&q=sort:newest ";
            end
            // In subjects or in full messages?
            fun = fun0
            if fun <> "!" & part(fun,$)=="!"  // search only in subjects of messages.
                s = "+subject:"
                fun = part(fun,1:$-1)
            else                   // search in subjects + bodies
                s = "+"
            end
            d = "!" // regexp delimiter
    
            // Authors part:
            [s0, s1, tmp, txt] = regexp(fun, "/^([a-z0-9_\-\.\s|\?\*áàâãäāăåéêèëēěėíîïóòôõöőōúùûŭũūůüűýŷÿÁÀÂÃÄĀĂÅÉÊÈËĒĚĖÍÎÏÓÒÔÕÖŐŌÚÙÛŬŨŪŮÜŰÝŶŸćĉčçċďđǵĜ ğġĥħĵķĺļłńňņŕřŗśŝšşťţŵźžżĆĈČĊÇĎĐ ĝĢĞĠĤĦĴĶĹĻŁŃŇŅŔŘŖŚŜŠŞŤŢŴŹŽŻ]+?)\>/i","o")
            txt = stripblanks(txt)
            if txt ~= ""
                authors = "from:(" + strsubst(txt,"|"," OR ")+") "
                fun = " " + part(fun, s1+1:$)
            else
                authors = ""
            end
    
            // Max age part:
            tmp = tokens(fun, "<")
            if size(tmp,1)>1
                dstart = getdate(getdate("s")-evstr(tmp($))*86400);
                dstart = msprintf("%4d%02d%02d", dstart(1),dstart(2), dstart(6)); 
                dend = getdate(); 
                dend = msprintf("%4d%02d%02d", dend(1),dend(2), dend(6)); 
                max_age = " date:[" + dstart + " TO " + dend + "] ";
                fun = strcat(tmp(1:$-1),"<")
            else
                max_age = ""
            end
            fun = stripblanks(fun)
    
            // Topics part
            if fun~=""
                if grep(fun,"&")==[] & sum(grep(fun, ["~" "(" ")"]))<2
                    fun = "("+fun+")"
                end
                tmp = uman_strsubst(fun, [ "[" "]" ],["\[" "\]"])
                tmp2 = ["%" "!" "?" "#" "[" "]" "-" "+" "*" "/" "^" "$" ]
                tmp = uman_strsubst(tmp, tmp2, "%"+dec2hex(ascii(tmp2)))
                // ":" must be protected with "\:" because of "subject:","author:","message:"
                tmp = uman_strsubst(tmp, ":", "%5C%3A")
                //
                tmp = uman_strsubst(tmp, [ "(" ")" ],["{" "}"])
                tmp = uman_strsubst(tmp, d+["\s*\&\s*\~\{(.+)\}" ..    // & ~{..}
                                            "\&?\s*\~\s*" ]+d,..    // & ~ | ~
                                           ["+NOT"+s+"{$1}" "+NOT"+s ], "r")
                tmp = uman_strsubst(tmp, d+["\s*\|\s*"]+d, ["+OR+"], "r")
                tmp = uman_strsubst(tmp, d+["\s*&\s*" "\{\s*?(.+?)\s*\}"]+d, [s "($1)"], "r")
                if grep(tmp,d+"^\"+["+NOT\+" s]+d,"r")==[]
                    tmp = s+tmp
                end
            else
                tmp = ""
            end
        end

        // Final gathering:
        tmp = authors + max_age + part(tmp, 2:$)
        MailingListsQuery = MailingListsQuery + tmp
        // DISPLAY
        uman_openURL(MailingListsQuery, noWebBrowserMsg)
        // QUIT
        return
    end
    // ========================================================================
    //
    // PREPROCESSING "b" pattern
    // --------------------------
    // The pattern may have from 1 to 4 parts a,b > c < d :
    // $component1|component2:author1|author2>topic1|topic2..<Ndays
    // my_reaction_emoji=thumbsup
    // Regression
    components = []
    authors = []
    urlQuery = ""
    tmp = strsplit(fun0,">")
    if size(tmp,1)>1
        trail = strcat(tmp(2:$), ">")
        tmp = stripblanks(strsplit(tmp(1), "/"))
        if size(tmp,1)==1 & tmp~="" then
            authors = stripblanks(tokens(tmp,"|"))
        else
            components = stripblanks(tokens(tmp(1),"|"))
            if size(components,1)==1 & components==""
                components = []
            end
            if size(tmp,1)>1
                authors = stripblanks(tokens(tmp(2),"|"))
                if size(authors,1)==1 & authors==""
                    authors = []
                end
            end
        end
        // LABELS:
        // + Regression
        // True labels including ":" : the ":" must be replaced with "%"
        compaliases = ..
        "atoms:ATOMS pkg manager,packages:ATOMS pkg manager," + ..
        "api:Hard coding,hardcoding:Hard coding,mexlib:Hard coding," + ..
        "batch:Terminal mode,cli:Terminal mode,terminal:Terminal mode," + ..
        "cacsd:CACSD,compile:Building Scilab,build:Building Scilab," + ..
        "containers:Lists cells structs," + ..
        "demos:Demonstrations," + ..
        "doc:Documentation pages,documentation:Documentation pages," + ..
        "desktop:Desktop - workspace,ide:Desktop - workspace," + ..
            "workspace:Desktop - workspace," + ..
        "edit:Scinotes,editor:SciNotes,scinotes:SciNotes," + ..
        "files:Files management,iodata:Files management," + ..
             "rw:Files management," + ..
        "figfiles:Graphics files," + ..
        "graphics:Graphic plotting," + ..
        "gui:Graphic interactions," + ..
        "help:Documentation pages," +..
        "helpsys:Documentation system,docsys:Documentation system," + ..
        "files:Files management," + ..
        "interaction:Graphic interactions,interactions:Graphic interactions," + ..
        "instal:Install,install:Install," + ..
        "internal:Internal maintenance,internals:Internal maintenance," + ..
        "maintenance:Internal maintenance," + ..
        "io:Input / output," + ..       // not documented (obsolete)
        "java:Java features," + ..
        "localization:Translation,locale:Translation," + ..
        "maths:Numerical Math,math:Numerical Math," + ..
        "m2sci:m2sci,matlab:m2sci," + ..
        "optim:Optimization,optimization:Optimization," + ..
        "os:OS functions," + ..
        "parser:Syntax / Interpreter,syntax:Syntax / Interpreter," + ..
        "parallel:Parallel computing," + ..
        "plotting:Graphic plotting,plot:Graphic plotting," + ..
        "preferences:Preferences," + ..
        "regression:Regression," + ..
        "startup:Startup crash," + ..
        "stat:Statistics,stats:Statistics,statistics:Statistics," + ..
        "string:String / Text,strings:String / Text,text:String / Text," + ..
        "sparse:Sparse,sparses:Sparse," + ..
        "tcl:TCL,tcltk:TCL," + ..
        "tests:Code performance,perfs:Code performance,quality:Code performance," + ..
        "translation:Translation," + ..
        "uicontrol:Graphic interactions," + ..
        "variables:Variables management,vars:Variables management," + ..
        "xcos:Xcos," + ..
        "xml:XML - HTML Files," + ..
        "onlinehelp:online%Help," + ..
        "onlineatoms:online%ATOMS";

//        "bugtracker:Bug tracker&product=Scilab websites," + ..
//        "bugzilla:Bug tracker&product=Scilab websites," + ..
//        "codereview:codereview&product=Scilab websites," + ..
//        "gerrit:codereview&product=Scilab websites," + ..
//        "fileexchange:file exchange&product=Scilab websites," + ..
//        "forge:forge&product=Scilab websites," + ..
//        "git:git&product=Scilab websites," + ..
//        "gitweb:git&product=Scilab websites," + ..
//        "wiki:wiki&product=Scilab websites," + ..
//        "website:website&product=Scilab websites," + ..
//        "ml:Mailing Lists&product=Scilab websites," + ..
//        "mailinglists:Mailing Lists&product=Scilab websites," + ..
//        "onlinehelp:Online Help&product=Scilab websites," + ..
//        "helponline:Online Help&product=Scilab websites," + ..
//        "helpweb:Online Help&product=Scilab websites," + ..
//        "webhelp:Online Help&product=Scilab websites";

        // Other flags
        compaliases = compaliases + ",bz:Bugzilla,bugzilla:Bugzilla"
        compaliases = matrix(tokens(compaliases,["," ":"]), 2, -1);
        compaliases(2,:) = strsubst(compaliases(2,:), "%", ":")
        definedComp = compaliases(1,:);
        if authors~=[]
            // if at least one of the authors is a defined component then all
            // are assumed to rather be components:
            tmp = members(convstr(authors), convstr(definedComp));
            if sum(tmp)>0
                tmp = components
                components = authors
                authors = tmp
            end
        end
        // Exploding aliases
        components = strcat(components,",")
        if components~=[]
            components = uman_strsubst("," + components + ",", "," + ..
                         compaliases(1,:) + ",", "," + compaliases(2,:) + ",")
            components = strsubst(components, "/", "%2F");
            components = tokens(components,",")
            //Input / output
            //Startup crash
        end
    else
        trail = tmp
    end
    // Components : label_name%5B%5D=""
    if components~=[] then
        tmp = "&label_name%5B%5D=" + components
        urlQuery = urlQuery + strcat(tmp) + "&"
    end
    // Authors
    // -------
    // We consider only the first author
    // If there is a Bugzilla component, former reports on Buzilla < 2022-10 are addressed
    // Then, the author is registered in the report's description with "by <author>"
    // Otherwise, the GET label is author_username=
    Bugzilla = %f
    if authors <> [] then
        if or(components=="Bugzilla")
            Bugzilla = %t
        else
            urlQuery = urlQuery + "&author_username=" + authors(1)
        end
    end
    // Topic // & max_age after last <
    tmp = strsplit(trail, "<")
    if size(tmp,1)==1
        fun0 = stripblanks(trail)
    elseif stripblanks(tmp($))==""
        fun0 = strcat(tmp(1:$-1),"<")
    end
    // ========================================================================

    // MERGING DEFAULT OPTIONS
    // -----------------------
    global umanDefaultCallOptions
    if isdef("umanDefaultCallOptions")
        dopt = convstr(umanDefaultCallOptions)
        [dopt, dLANG] = uman_splitOptionsLang(dopt)
        // "b"|"w"|"@"|"r" not supported in defaults
        dopt = uman_strsubst(dopt,["b" "w" "@" "r"],"")
        // Calling options p,d,e,h,b,a overrides any a|s default ones
        if grep(options,["p" "d" "e" "h" "a" "u"])~=[] & grep(dopt,["a" "s"])~=[]
            dopt = uman_strsubst(dopt, ["a" "s"], "")
        end
        // Calling options p,d,e,h,b,a,s overrides default u
        if grep(options,["p" "d" "e" "h" "a" "s"])~=[] & grep(dopt,"u")~=[]
            dopt = uman_strsubst(dopt, "u", "")
        end
        // Calling option u overrides any p,d,e,h,b,a default ones
        if grep(options,"u")~=[] & grep(dopt,["p" "d" "e" "h" "a"])~=[]
            dopt = uman_strsubst(dopt, ["d" "e" "h" "a"], "")
        end
        options = options+dopt
    end

    // FINAL LANGUAGE
    // --------------
    if LANG==""
        if ~isdef("dLANG","l") | dLANG==""
            LANG = part(getlanguage(), 1:2)
        else
            LANG = dLANG
        end
    end

    // OPTIONS ANALYSIS
    // ----------------
    fun = fun0
    if grep(options, "a")~=[] then
        options = strsubst(options,"a","apdeh")
        // "a" will still be needed in case of "b" mode
    end
    fun = uman_strsubst(fun,"/^%/" ,"percent","r")
    // This substitution will have to be cancelled in "b" mode before calling
    //  the bug page. Nevertheless, it is required to search the right page
    //  to determine the related module (or scilab) and the related bug page
    //  to be opened.

    // option b > w,g,@,s,u
    if grep(options, "b")~=[] then
        options = uman_strsubst(options, ["w" "g" "@" "s" "u"], "")
    end
    // option w > @,s,u
    if grep(options, "w")~=[] then
        options = uman_strsubst(options, ["@" "s" "u"], "")
    end
    // option g > @,u
    if grep(options, "g")~=[] then
        options = uman_strsubst(options, ["@" "u"], "")
    end
    // option "u" (usage) kills options p,d,e,h,a,s
    if grep(options, "u")~=[] then
        options = uman_strsubst(options, ["p" "d" "e" "h" "a" "s"], "")
    end
    bugOnWeb = grep(options,"b")~=[]

    // SETTING THE LANGUAGE
    // --------------------
    supportedLANG = [ "en_US" "fr_FR" "ja_JP" "de_DE" "pt_BR" "ru_RU" "zh_CN" "fa_IR"]
    i = grep(supportedLANG, "/^"+LANG+"_/", "r")
    if i==[] then
        LANG = "en_US"
    else
        LANG = supportedLANG(i)
    end

    // REDIRECTIONS for Symbols. Translation for "%" in fun names
    // ----------------------------------------------------------
    if ~bugOnWeb then
        fun = uman_strsubst(fun,["/\{/" "/\}/"],["[" "]"],"r")
        sub = "assignation,great,  > ,  < ,   [    ,   ]    ,  :  ,   //   ,"
        ids = "insertion  ,less ,less,less,brackets,brackets,colon,comments,"

        sub = sub+"    <>    ,   ~=     ,   <=     ,   >=     ,"
        ids = ids+"comparison,comparison,comparison,comparison,"

        sub = sub+"  =  ,    ==    ,"
        ids = ids+"equal,comparison,"
        // bug with these ones, due to Scilab parser.
        // Must be set within quotes: uman "=="

        sub = sub+"  &   ,  |  , ~ ,   $  , . , []  ,"
        ids = ids+"and_op,or_op,not,dollar,dot,empty,"
            // % automaticaly replaced with percent

        sub = sub+"  -  ,  + ,  * , .* ,  /  , ./  ,  /. , .^  , **  , ^ , .*.,    \    ,"
        ids = ids+"minus,plus,star,star,slash,slash,slash,power,power,hat,kron,backslash,"

        sub = sub+"    \.     ,     *.    ,    ./.    ,    .\.    ,"
        ids = ids+"overloading,overloading,overloading,overloading,"

        sub = sub+"  '' ,    ;    "
        ids = ids+"quote,semicolon"

        sub = sub+""
        ids = ids+""
        sub = [ stripblanks(strsplit(sub, ",")) ; "," ]
        ids = [ stripblanks(strsplit(ids, ",")) ; "comma" ]
        i = find(fun==sub)
        if i~=[] then
            fun = ids(i(1))
        end
        if grep(fun, " ") then
            fun = strsubst(fun, " ", "_")
        end
    end

    // OTHER INITIALISATIONS
    // ---------------------
    // Case sensitivity for the extraction
    if convstr(fun)==fun then
        casse = "-C"    // case-insensitive
    else
        casse = ""      // case-sensitive
    end
    // Setting the unzip application vs OS to extract pages from JAR archives
    if getos() == "Windows" then
        unzip  = pathconvert(SCI+"\tools\zip\") + "unzip.exe "+casse+" -o -q "
        //unzip2 = pathconvert(SCI+"\tools\gzip\") + "gzip.exe "
    else
        unzip = "unzip " + casse + " "
    end

    // Miscellaneous init
    fs = filesep()
    unzippedFile = ""        // = flag for "File Not Found"

    // Directories storing unzipped HTML files
    // ---------------------------------------
    //  Root directory
    manDir = pathconvert(SCIHOME + "\uman\")
    if fileinfo(manDir)==[] then
        mkdir(manDir)
    end
    // Main directory for Scilab HTML extracted pages
    manScilabDir = pathconvert(manDir + "HTML_pages_Scilab\")
    if fileinfo(manScilabDir)==[] then
        mkdir(manScilabDir) // only once. Scilab files don't need to be refreshed
    end
    manScilabDir = getshortpathname(manScilabDir)
    // Main directory for ATOMS and other contribs HTML extracted pages
    manAtomsDir = pathconvert(manDir + "HTML_pages_Contrib\")
    if fileinfo(manAtomsDir)==[] then
        mkdir(manAtomsDir)
    end
    // Main directory for modules out of ATOMS and contrib HTML extracted pages
    manOtherDir = pathconvert(manDir + "HTML_pages_Other\")
    if fileinfo(manOtherDir)==[] then
        mkdir(manOtherDir)
    end
    // Refresh option
    // => deleting all extracted HTML pages in manAtomsDir & manOtherDir
    if grep(options, "r")~=[] then
        rmdir(manAtomsDir,"s")
        mkdir(manAtomsDir)
        rmdir(manOtherDir,"s")
        mkdir(manOtherDir)
    end
    manAtomsDir = getshortpathname(manAtomsDir)
    manOtherDir = getshortpathname(manOtherDir)

    // Listing JAR files: Scilab + ATOMS/Contrib(all_users) + ATOMS/Contrib(user)
    // ==========================================================================
    // We need this part also for bugOnWeb: if the pattern matches an external
    //  function (e.g. from ATOMS), the external module must be identified
    //  in order to target the right online bug report place

    function flist = dirjar(path)
        // Returns the column of paths to ALL JAR files (not only for LANG or en_US)
        // path must be ended with filesep()
        if isdir(path) then
            if getos()=="Windows" then
                flist = unix_g("dir /s /b """+path+"scilab_*_help.jar"" 2> nul")
            elseif getos()=="Linux"
                flist = unix_g("find "+path+" -name scilab_*_help.jar 2> /dev/null")
            else
                flist = unix_g("find 2>/dev/null "+path+ "scilab_*_help.jar")
            end
            if flist=="" then
                flist = []
            end
        else
            flist = []
        end
    endfunction

    jarListFile = manDir + "jarListe.txt"
    ATOMSisChanged = uman_isATOMSchanged();
    i = fileinfo(jarListFile)
    // If the list of JAR files does not exist, or if the "r" (reload / refresh)
    //  option is used, we rebuild and record it and re-extract jhelpmap.jhm files :
    reScanResources = ( i==[] | ATOMSisChanged | grep(options,"r")~=[] );
    if reScanResources then
        if getscilabmode()~="NWNI" then
            hpb = progressionbar("Rescanning resources... Please wait");
        end
        // Building the list of JAR files
        // ------------------------------
        // Let's GO:
        jarList = []
        // Scilab trunk:
        path = pathconvert(pathconvert(SCI)+"modules\helptools\jar\")
        jarList = [jarList ; dirjar(path)]
        // Contrib (ATOMS and others) : ALL USERS
        path = pathconvert(pathconvert(SCI)+"contrib\")
        jarList = [jarList ; dirjar(path)]
        // USER : ATOMS
        path = pathconvert(pathconvert(SCIHOME)+"atoms\")
        jarList = [jarList ; dirjar(path)]
        // USER : other contribs
        path = pathconvert(pathconvert(SCIHOME)+"contrib\")
        jarList = [jarList ; dirjar(path)]
        // OTHER PATHS provided in umanUserPaths
        global umanUserPaths
        if umanUserPaths~=[] & type(umanUserPaths)==10
            for p = umanUserPaths(:)'
                if isdir(p)
                    jarList = [jarList ; dirjar(pathconvert(p))]
                else
                    msg = _("uman", "WARNING : uman: Preferences => Other paths:\n\t''%s''\n\t path is not valid.\n")
                    mprintf(msg, p)
                end
            end
        end

        // Recording the list:
        csvWrite(jarList, jarListFile, ",")
    else
        jarList = mgetl(jarListFile)
    end

    // Various initializations
    // -----------------------
    module = ""
    clang = getlanguage()

    // Extraction of jhelpmap.jhm files
    // ================================
    // as well needed by the "b" option (for forge's tickets)

    // Directory storing jhelpmap.jhm files renamed as moduleName_la_RG.jhm
    //  or scilab_la_RG.jhm, where la_RG is the language_ReGion code (en_US, etc)
    jhmListFile = manDir + "jhmListe.txt"
    mapDir = manDir + "jhelpMaps" + fs
    if ~isdir(mapDir) then
        mkdir(mapDir)
        mkdir(mapDir+"scilab"+fs) // subdir for fixed scilab_la_RG.jhm files
        mkdir(mapDir+"contrib"+fs)// .. for ATOMS & contrib scilab_la_RG.jhm files
        mkdir(mapDir+"other"+fs)  // .. for jhm files of ungistered libraries
    elseif ~isdir(mapDir+"contrib") then  // for first use of uman 3.1
        mkdir(mapDir+"contrib"+fs)
        mkdir(mapDir+"other"+fs)
    else
        if grep(options, "r")~=[] then
            mdelete(mapDir+"contrib"+fs+"*.jhm")
            mdelete(mapDir+"other"+fs+"*.jhm")
        end
    end
    // ..............................................................
    function jhmListe = uman_extract_jhelpMaps_from_jars(jarList, other)
        // jarList: column of strings = paths to the jar files
        // other: boolean. %T if the jars are about a module out of ATOMS & contrib
        //
        // From the list of selected JAR files, for each file, extracts its
        //  jhelpmap.jhm in a common directory, and renames it according to the
        //  module AND language tag.
        // "unzip" includes the options -C: case-insensitive -o: overwrite -q: quiet
        //
        // External variables used:
        // fs, unzip, manScilabDir, manAtomsDir, manOtherDir

        if ~isdef("other","l"), other = %F; end
        pattern = pathconvert("\modules\helptools\jar\scilab_", %f)
        jhmListe = []
        for jar = jarList'
            // Module name related to the .jar file = ?
            if grep(jar, pattern)~=[]
                module = "scilab"
            else
                // The name of the module (such as stored in %helps(:,2))
                //  is retrieved through the file name ~\etc\modulename.start
                //  tmp = fullpath(fileparts(jar,"path")+fs+"..\etc\*.start") // 6.1.1 bug
                tmp = fullfile(fileparts(jar,"path"), "..\etc\*.start")
                tmp = dir(tmp)
                startfile = tmp.name(1)
                module = basename(startfile)
            end
            if module~="" & module~=[]
                lang = uman_strsubst(jar, "|.*?_([a-z]{2}_[A-Z]{2})_help.jar|", "$1", "r")
                subDir = "scilab_"+lang+"_help"+fs
                zippedMap = subDir + "jhelpmap.jhm"
                if getos()=="Windows"
                    jar = getshortpathname(jar)    // strips blanks from the path
                end
                //command = unzip + """"+jar + """ """ + zippedMap + """ -d """+manDir+""""
                command = unzip + jar + " " + zippedMap + " -d "
                if module=="scilab"
                    command = command + manScilabDir
                    unzippedMapFrom = manScilabDir + subDir + "jhelpmap.jhm"
                    unzippedMapTo = "scilab"+fs+module+"_"+lang+".jhm"
                elseif ~other
                    command = command + manAtomsDir
                    unzippedMapFrom = manAtomsDir + subDir + "jhelpmap.jhm"
                    unzippedMapTo = "contrib"+fs+module+"_"+lang+".jhm"
                else
                    command = command + manOtherDir
                    unzippedMapFrom = manOtherDir + subDir + "jhelpmap.jhm"
                    unzippedMapTo = "other"+fs+module+"_"+lang+".jhm"
                end
                status = host(command);
                if isfile(unzippedMapFrom)
                    [s, m] = movefile(unzippedMapFrom, mapDir+unzippedMapTo);
                    jhmListe = [jhmListe ; [module unzippedMapTo jar ]];
                end
            end
        end
    endfunction
    // ..............................................................
    if reScanResources then
        fs = filesep()
        jhmListe = uman_extract_jhelpMaps_from_jars(jarList)

        // Removing duplicates .jhm: the last is kept (has overwritten previous ones)
        jhmListe = jhmListe($:-1:1,:)
        [tmp, k] = unique(jhmListe(:,2))
        jhmListe = jhmListe(gsort(k,"g","i"),:)
        csvWrite(jhmListe, jhmListFile, ",")

        // Closing the wait bar:
        if isdef("hpb","l")
            close(hpb)
        end
    else
        jhmListe = csvRead(jhmListFile,",",[],"string")
    end

    // Building 2 lists of jhmHelpMaps, ordered by lang:
    //   1) for Scilab   2) for external modules
    // =================================================
    function k = uman_orderJhmHMapsByLang(c, external?)
        // c = jhmListe(:,2) : column of module_##_##.jhm names
        // external? : boolean
        //       %T whether only ATOMS or contrib modules are considered
        //       %F whether only native Scilab is considered
        // LANG and clang external contextual variables are used.
        //
        // The language has priority:
        // Chosen language (or default)
        if external? then
            pattern = "!^(contrib|other)\"+fs+"[^\"+fs+"]+_%s\.jhm!"
            // Chosen language (or default)
            k = grep(c, strsubst(pattern, "%s", LANG),"r")
            // Otherwise: Session's language
            if clang~=LANG
                k = [k grep(c, strsubst(pattern, "%s", clang),"r")]
            end
            // Otherwise, finally: en_US
            if LANG~="en_US" & clang~="en_US"
                k = [k grep(c, strsubst(pattern, "%s", "en_US"),"r") ]
            end
        else
            k = find(c=="scilab"+fs+"scilab_"+LANG+".jhm")
            if clang~=LANG
                k = [ k find(c=="scilab"+fs+"scilab_"+clang+".jhm") ]
            end
            if LANG~="en_US" & clang~="en_US"
                k = [ k find(c=="scilab"+fs+"scilab_en_US.jhm") ]
            end
        end
    endfunction

    // Scilab list:
    k = find(jhmListe(:,1)=="scilab")
    tmp = jhmListe(k, :)
    k2 = uman_orderJhmHMapsByLang(tmp(:,2), %f)
    jhmScilab = jhmListe(k(k2),:)

    // List for ATOMS & contribs:
    k = find(jhmListe(:,1)~="scilab")
    tmp = jhmListe(k, :)
    if tmp~=[] then
        k2 = uman_orderJhmHMapsByLang(tmp(:,2), %t)
        jhmModules = jhmListe(k(k2),:)
    else
        jhmModules = []
    end

    // SEARCHING FOR THE RIGHT HELP TARGET (module, page, local macro, web pointer)
    // ====================================
    htmlName = ""
    localMacro = ""
    webId = ""      // Id for an ATOMS, a FE, or another external ressource
    jhmLine = []
    if casse=="-C"
        grepCase = "i"
    else
        grepCase = ""
    end

    if fun~="" & grep(fun, ["|" "/" "\" "*" ":" "[" "("])==[]
        // Priorities: Scilab > external modules > local macro > web pointer
        // -----------------------------------------------------------------
        funR = ""
        if grep(options, "x")==[]
            // PRIORITIES for Soft mormal mode AND "x"==[] :
            // (Shortcuts)Scilab
            // (ex2in) Scilab
            // Modules
            // Unregistered libraries
            // local
            // (ex2ex) Modules
            // (ex2key) Scilab
            [fun1s, optmp] = uman_get_equiv(fun, options, "shortcuts")
            [htmlName, jhmLine] = uman_findItemInJHelpMaps(fun1s, jhmScilab, mapDir, grepCase)
            if htmlName==""
                // (ex2in) Scilab
                [fun2s, optmp] = uman_get_equiv(fun, options, "ex2in")
                if fun2s~=fun1s
                    [htmlName, jhmLine] = uman_findItemInJHelpMaps(fun2s, jhmScilab, mapDir, grepCase)
                    if htmlName~=""
                        funR = fun2s, options = optmp
                    end
                end
            else
                funR = fun1s, options = optmp
            end

            // ATOMS or contrib modules
            if htmlName=="" & jhmModules~=[]
                try
                    [htmlName, jhmLine] = uman_findItemInJHelpMaps(fun, jhmModules, mapDir, grepCase)
                end
                if htmlName ~= ""
                    funR = fun
                end
            end

            // Unregistered loaded library
            if htmlName==""
                tmp = whereis(fun)
                if tmp<>[] & tmp<>"script" & type(evstr(tmp))==14 then
                    [tmp,p] = libraryinfo(tmp); // path to the lib file
                    tmp = tokens(p, ["/" "\"])
                    // Getting the root path of the external module
                    while tmp($)<>"macros", tmp($)=[], end // in case of subdirs
                    tmp($) = []
                    p = strcat(tmp,filesep());
                    p = p + filesep() + "jar";
                    modJars = dirjar(getshortpathname(pathconvert(p)))
                    jhmListe = uman_extract_jhelpMaps_from_jars(modJars,%T)
                    k = uman_orderJhmHMapsByLang(jhmListe(:,2), %t)
                    jhmListe = jhmListe(k,:)
                    try
                        [htmlName, jhmLine] = uman_findItemInJHelpMaps(fun, jhmListe, mapDir, grepCase)
                    end
                    if htmlName ~= ""
                        funR = fun
                    end
                end
            end

            // Local
            if htmlName==""
                localMacro = uman_find_localMacro(fun)
                if localMacro~=""
                    funR = fun
                end
            end
            // (ex2ex) Modules
            if htmlName=="" & localMacro=="" & jhmModules~=[]
                [fun2m, optmp] = uman_get_equiv(fun, options, "ex2ex")
                if fun2m~=fun
                    [htmlName, jhmLine] = uman_findItemInJHelpMaps(fun2m, jhmModules, mapDir, grepCase)
                    if htmlName~=""
                        funR = fun2m, options = optmp
                    end
                end
            end
            // RESULTS: If htmlName=="" & localMacro=="" then funR==""
            //  otherwise funR~=""

        else
            // "x": ------------------------------------------------------------
            // PRIORITIES for Soft normal mode AND "x" :
            // Modules
            // local user definition
            // (faux-amis)(ex2in) Scilab
            // (ex2ex) Modules
            // local
            // (shortcuts) Scilab
            // (ex2key) Scilab

            // Modules
            if jhmModules~=[]
                [htmlName, jhmLine] = uman_findItemInJHelpMaps(fun, jhmModules, mapDir, grepCase)
            end
            // Local definition
            if htmlName=="" & or(whereis(fun)=="script")
                head_comments(fun)
                return
            end
            // (faux-amis)(ex2in) Scilab
            if htmlName==""
                [fun1s, optmp] = uman_get_equiv(fun, options, "exin2in")
                [fun1s, optmp] = uman_get_equiv(fun1s, options, "ex2in")
                [htmlName, jhmLine] = uman_findItemInJHelpMaps(fun1s, jhmScilab, mapDir, grepCase)
                if htmlName~=""
                    funR = fun1s, options = optmp
                end
            end
            // (ex2ex) Modules
            if htmlName=="" & jhmModules~=[]
                [fun1m, optmp] = uman_get_equiv(fun, options, "ex2ex")
                [htmlName, jhmLine] = uman_findItemInJHelpMaps(fun1m, jhmModules, mapDir, grepCase)
                if htmlName~=""
                    funR = fun1m, options = optmp
                end
            end
            // local
            if htmlName==""
                localMacro = uman_find_localMacro(fun)
                if localMacro~=""
                    funR = fun
                end
            end
            // (shortcuts) Scilab
            if htmlName=="" & localMacro==""
                [fun2s, optmp] = uman_get_equiv(fun, options, "shortcuts")
                if fun2s~=fun1s
                    [htmlName, jhmLine] = uman_findItemInJHelpMaps(fun2s, jhmScilab, mapDir, grepCase)
                    if htmlName~=""
                        funR = fun2s, options = optmp
                    end
                end
            end
        end     // if "x"

        // (ex2key) Scilab (common to "x" & ~"x" at the end):
        if htmlName=="" & localMacro==""
            [fun3, optmp] = uman_get_equiv(fun, options, "ex2key")
            funR = fun3, options = optmp
        end
    fun = funR
    end

    // Module identification & informations (atoms, forge, website)
    // ------------------------------------------------------------
    otherModule = %F    // for unregistered libraries loaded in Session
    module = ""
    if jhmLine~=[] then
        module = jhmLine(1)
        otherModule = grep(jhmLine(2),"other"+filesep())
    end

    // List of installed ATOMS modules
    atomsInstalled = atomsGetInstalled()
    if atomsInstalled~=[] then
        atomsInstalled(:,4) = getshortpathname(pathconvert(atomsInstalled(:,4)))
    end

    // Sometimes the module id from the jhm is NOT the true one.
    // Let's check against the id in the atoms list
    if and(module<>["" "scilab"]) then
        [?,k] = grep(module, atomsInstalled(:,1))
        if ?==[]
            k = grep(atomsInstalled(:,1), module)
        end
        if k~=[]
            module = atomsInstalled(k,1)
        end
    end

    //
    AF = uman_get_atoms_forge()
    AFid = AF(:,2)
    webRef = []
    if module~="scilab"  then
        k = []
        if module~="" then
            k = find(AFid==module)
            if k==[]
                k = find(convstr(AFid)==convstr(module))
            end
        else
            if fun==convstr(fun)
                k = find(convstr(AFid)==fun)
            else
                k = find(AFid==fun)
            end
            if k==[]    // We then try after redirection:
                fun1e = uman_get_equiv(fun, options, "ex2ex")
                if fun1e==convstr(fun1e)
                    k = find(convstr(AFid)==fun1e)
                else
                    k = find(AFid==fun1e)
                end
            end
        end
        if k~=[]
            webRef = AF(k,:)  // 1: type  2: module_id  3: forge_id  4: website
            module = webRef(2)
        end
    end

    isATOMSinstalled = (find(atomsInstalled(:,1)==module)~=[])

    // Now we have:
    // -----------
    // htmlName: the HTML reference to extract (if found)
    // module  : the name of the external module (if relevant)
    // isATOMSinstalled : boolean about the module (if any)
    // localMacro: the name of the local macro (if it exists)

     // For "b", required infos are now available => DISPLAY

    // ========================================================================

    // GITLAB | TICKET / FORGE | WEBSITE
    // =================================
    // Default place for Scilab or mixed items:
    GitlabSearch = "https://gitlab.com/scilab/scilab/-/issues/?first_page_size=50&sort=created_date&state=%s"
    global umanListOnlyUnresolvedBugs
    if ~umanListOnlyUnresolvedBugs | grep(options,"a")<>[] then
        GitlabSearch = msprintf(GitlabSearch, "all")
    else
        GitlabSearch = msprintf(GitlabSearch, "opened")
    end
    // £ will be replaced with the item name
    BugReport = ""
    if bugOnWeb

        // Bug number provided: no other infos required => Display
        // -------------------------------------------------------
        if grep(fun0,"/^[0-9]+$/","r")==1
            if  evstr(fun0)>76  // item = bug number
                // There are some holes > 76
                // (all reports about websites deleted in 2022 when moving to Gitlab)
                url = "https://gitlab.com/scilab/scilab/-/issues/"+fun0
                uman_openURL(url, noWebBrowserMsg)
                return
            end
        end
        // Setting the right bug report place for externals
        // ------------------------------------------------
        // ATOMS modules registered for "uman"
        if module=="removed"
            // If the item is a removed one and the "removed" module is installed,
            // Bugzilla must still be targeted instead of the "removed" ATOMS page.
            fun = fun0  // To be reviewed & improved
        else
            if module~="scilab" & webRef~=[] then
                // If the external module is an ATOMS with a public forge:
                // Tickets are on it
                if webRef(3)~=""
                    BugReport = "https://gitlab.com/scilab/forge/"+webRef(3)+"/-/issues/"
                    GitlabSearch = BugReport + "?first_page_size=50&sort=created_date&state=%s"
                    if grep(options,"a")<>[]
                        GitlabSearch = msprintf(GitlabSearch, "all")
                    else
                        GitlabSearch = msprintf(GitlabSearch, "opened")
                    end
                    if fun ~= "" | fun0 ~= ""
                         GitlabSearch = GitlabSearch + "&in=TITLE&search="
                         if htmlName~=""
                            GitlabSearch = GitlabSearch + fun
                         else
                            GitlabSearch = GitlabSearch + fun0
                         end
                     end

                else
                    // If in ATOMS (without forge): => we target ATOMS comments
                    if webRef(1)=="a"
                        tmp = "https://atoms.scilab.org/toolboxes/"+module+"/#leave_a_comment"
                    elseif webRef(1)=="fe"  // FileExchange
                        tmp = stripblanks(webRef(4))+"/#leave_a_comment"

                    elseif webRef(1)=="fo"  // Forge without ATOMS
                        tmp = "https://gitlab.com/scilab/forge/" + ..
                            stripblanks(webRef(2))+"/-/issues/?first_page_size=50&sort=created_date&state=opened"
                        if fun ~= "" | fun0 <> ""
                            GitlabSearch = GitlabSearch + "&in=TITLE&search=£"
                            if htmlName~=""
                                tmp = tmp + fun
                            else
                                tmp = tmp + fun0
                            end
                        end
                    else // Otherwise we target verbatim the indicated website
                        tmp = stripblanks(webRef(4))
                    end
                    BugReport = tmp
                    GitlabSearch = tmp
                end

            // Installed ATOMS module BUT unregistered for uman:
            elseif isATOMSinstalled
                tmp = "https://atoms.scilab.org/toolboxes/"+module+"/#leave_a_comment"
                BugReport = tmp
                GitlabSearch = tmp

            // Other Contrib modules
            elseif module~="scilab" & htmlName~=""
                GitlabSearch = ""
            end
        end

        if GitlabSearch~=""  then
            fun = strsubst(fun, "/^percent/", "%", "r")
            //symbols = "><[]:<>~=<=>==&|$.**-+././.^.*.\.'';"
            if grep(fun,"/")~=[]
                delim = "!"
            else
                delim = "/"
            end
            [prepend, append] = ("","")
            if grep(("a":"z")+("A":"Z")+"%",part(fun,1))~=[]
                prepend = "(^|[^a-zA-Z_])"
            end
            if fun~="" & grep(("a":"z")+("A":"Z")+("0":"9"),part(fun,$))~=[]
                append = "([^0-9a-zA-Z_%3B]|$)"
              // ([^0-9a-zA-Z_%3D%3B]|$) => ([^0-9a-zA-Z_%3B]|$), otherwise
              //  'uman figure_size b' does not detect  '... a.figure_size=v...'
            end
//            tmp = uman_strsubst(fun, ..
//                delim+["\\"   "&"   "\*"  "\."  "\+"    "\-"  "\:"  "\|\|"   "\]"  "\["  "\("  "\)"  "\$"    "\^"   "/"   "\?"    "\#"    "\;"    """"]+delim, ..
//                      ["\\\\" "%26" "\\*" "\\." "\\%2B" "\\-" "\\:" "\\|\\|" "\\]" "\\[" "\\(" "\\)" "\\%24" "\\^"  "%2F" "\\%3F" "\\%23" "\\%59" "\\%22"], "r")
//            if tmp==fun
//                tmp = fun0
//            end
//            if tmp==""
//                tmp = ".*"
//            end
            if Bugzilla & authors<>[]
                tmp = "by " + authors(1)+" "
                fun0 =  tmp + fun0
                fun = tmp + fun
            end
            if fun~=""
                if regexp(GitlabSearch, "/gitlab\.com\/scilab\//") <> [] & ..
                   regexp(GitlabSearch, "/&search=/")==[]
                    GitlabSearch = GitlabSearch + "&in=TITLE&search=£"
                end
                url = strsubst(GitlabSearch,"£", fun0)
            else
                url = strsubst(GitlabSearch,"£", "")
            end
            url = url + urlQuery
            if Bugzilla & authors <> []
                url = strsubst(url, "&in=TITLE", "")
            end
            uman_openURL(url, noWebBrowserMsg)
        else
            // NO SITE TO SEE BUGS....
        end
        return
    end

    // ========================================================================

    // Targetting ONLINE help page (or site)
    // =====================================
    onlinePage = %f;
    printWeb = grep(options,"w")~=[] | (grep(options0,"w")~=[] & unzippedFile=="")
    if printWeb
        url = "https://help.scilab.org/docs/current/"
        supported_online= ["en_US" "fr_FR" "pt_BR" "ru_RU" "ja_JP"]
        if and(LANG~=supported_online)
            langH = "en_US"
        else
            langH = LANG
        end
        // Is it a removed item?
        // ---------------------
        removed = uman_getRemoved(fun0);  // [name replacement version]
        msgRem = ""
        if removed~=[]  // Yes
            msgRem = _("uman", "''%s'' was REMOVED after Scilab %s")
            msgRem = msprintf(msgRem, fun0, removed(1,3))
            if strcmp(removed(1,3),"5.3")<0  // no online page
                if removed(1,2)~=""  // but a replacement => we display its page
                    url = url + langH + "/" + removed(1,2) + ".html"
                    tmp = _("uman", ". ''%s'' replacement displayed.")
                    msgRem = msgRem + msprintf(tmp, removed(1,2))
                else                // no replacement => "removed" ATOMS page
                    url ="https://atoms.scilab.org/toolboxes/removed";
                end
            else    // Online archives
                onlinePage = %t;
                msgRem = msgRem + _("uman", " => Display archive")
                if removed(1,2)~=""
                    tmp = _("uman", ". Replaced with ''%s''")
                    msgRem = msgRem + msprintf(tmp, removed(1,2))
                end
                url = "https://help.scilab.org/docs/"+removed(1,3) + ..
                      "/"+ langH + "/" + removed(1,1)+".html"
            end
        // Does the feature (out of removed ones) have a web reference?
        // ------------------------------------------------------------
        elseif webRef~=[]
            // ATOMS module => open the ATOMS page
            if webRef(1)=="a"
                url = "https://atoms.scilab.org/toolboxes/"+webRef(2)

            // FileExchange => open the FE page
            elseif webRef(1)=="fe"
                url = webRef(4)

            // Forge with no ATOMS => open the Forge home page
            elseif webRef(1)=="fo"
                url = "https://gitlab.com/scilab/forge/"+webRef(2)
            // Other external resource
            else
                url = webRef(4)
            end
        elseif or(module==["" "scilab"])
            // url = "https://help.scilab.org/docs/current/"
            supported_online= ["en_US" "fr_FR" "pt_BR" "ru_RU" "ja_JP"]
            if and(LANG~=supported_online)
                langH = "en_US"
            else
                langH = LANG
            end
            // if unzippedFile~=""
            if htmlName ~= ""
                // url = url+langH+"/"+fun+".html"
                // url = url+langH+"/"+basename(unzippedFile)+".html"
                // For Type, Equal, etc: URL(web) ~= URL(jHelpMap):
                if grep(htmlName, "/.+\-[0-9a-f]{31,32}\.html/","r")
                    htmlName = strsubst(htmlName, "/\-[0-9a-f]{31,32}/","","r")
                end
                url = url + langH + "/" + htmlName
            else
                // query example: "cholesky+factorization"+AND+factors
                url = url + langH + "/search/" + fun0
            end
        end
        if msgRem~=""
            mprintf(msgRem+"\n")
        end
        if url~=""
            uman_openURL(url, noWebBrowserMsg)
        else
            disp(_("uman", "NO WEB REFERENCE AVAILABLE"))
        end
        // We don't return, because the "w" allows also "g"
    end

    // ========================================================================

    // Setting the flag allowing to target the console
    // ...............................................
    //  - whatever is the "g" status (can duplicate the output)
    printConsole = (grep(options,["w" "@" "b"])==[] | ..
                   (grep(options,"w")~=[] & module~="scilab" & ~onlinePage ))
                   // no online page is available for external modules
                   // => prints it in the console instead

    if printConsole & grep(options,"j")==[]
        diary([], "pause") // https://gitlab.com/scilab/scilab/-/issues/12551
    end
    noContent = %f  // set it to %t if [pattern] has no dedicated page

    // HTML Page's extraction, if needed and not yet in cache
    // ------------------------------------------------------
    if htmlName~="" then
       jar = jhmLine(3)  // JAR file that htmlName belongs to
       [tmp, tmp, tmp, lang] = regexp(jhmLine(2), "|_(.._..)\.jhm|")
       subDir = "scilab_"+lang+"_help"
       // is the HTML of this jar unzipped ?
       zippedFile = subDir + fs + htmlName
        if module=="scilab"
            outDir = manScilabDir   // ended with /
        elseif ~otherModule         // mixed ATOMS and other contribs are mixed
            outDir = manAtomsDir
        else
            outDir = manOtherDir    // unregistered libraries out of contribs
        end
        unzipped = outDir + subDir + fs + htmlName
       if isfile(unzipped) & grep(options, "r")==[]
           unzippedFile = unzipped
       else
           if isfile(unzipped)
               deletefile(unzipped)
           end
           if jar~=[]
               // We extract the HTML file:
               command = unzip + jar + " " + zippedFile + " -d " + outDir
               status = host(command)
               if status==0 // file found and extracted
                   unzippedFile = unzipped
               end
           else
               status = 9   // zip file not found
           end
       end
    end
    // -------------------------------------------------------
    global %helps
    helpBrowserWillBeCalled = getscilabmode()=="STD" & grep(options,"g")~=[] & ..
              ( module=="scilab" | ..
                ( isATOMSinstalled & atomsIsLoaded(module) ) | ..
                find(convstr(%helps(:,2))==convstr(module))~=[] ..
              )
    // -------------------------------------------------------
    printMissingOnline = grep(options,"w")~=[] & ..
            module~="" & module~="scilab" & webRef(1)=="a" & ~onlinePage;
    // -------------------------------------------------------

    // The required help file exists...
    // --------------------------------
    if unzippedFile~=""       // we go on on processing
        if printMissingOnline & grep(options0, ["p" "d" "e" "h"])==[]
            options = "pdeh"+options // The full page must be printed
        end
        r = uman_parse_html2sections(unzippedFile, fun, LANG)
        r.fun = fun0

        // Targetting the console
        // ......................
        msg2 = ""
        if  printConsole & ~helpBrowserWillBeCalled
            if grep(options,"c")~=[] then
                clc()

           // separating zone
            elseif grep(options,"u")==[]
                tmp = lines()
                write(%io(2), part("=", ones(1,tmp(1))))
            end
            // item's page
            // - - - - - -
            msg0 = ""
            if grep(options, "s")==[]
                // Display of the proper help page
                uman_disp(r, options)

            // Parent summary
            // - - - -- - - -
            else //pause
                tmp = strsplit(unzippedFile, fs)
                zippedFile = tmp($-1) + fs + r.parentSummary + ".html"
                tmp = strsplit(unzippedFile, fs)
                outDir = strcat(tmp(1:$-2), fs)
                unzippedFile = fileparts(unzippedFile,"path") + fs + r.parentSummary + ".html"

                if fileinfo(unzippedFile)==[]
                    command = unzip + jar + " " + zippedFile + " -d " + outDir
                    status = host(command)  // status==0  <=> OK
                end

                // Parsing the summary page ++++++++++++++++++++++++++
                html = mgetl(unzippedFile)
                search  = [ "<html>"
                            "<body>"
                            "<head>.*?</head>"
                            "<div class=""manualnavbar"">.*?</div>"
                            "</body>"
                            "</html>"
                            "<br />"
                            ]
                search = "|" + search + "|"
                replace = ["<div>" "" "" "" "" "</div>" ascii(10) ]
                html2 = uman_strsubst(strcat(html, " "), search, replace, "r")
                tmp = xmlReadStr(html2)
                simpleContent = %t

                // If language = ja|zh: we must assess the fraction of content in
                // in japanese|chinese vs latin, in order to tune the average
                // character width used to wrap long lines
                lang = part(LANG, 1:2)
                if or(lang==["ja" "zh"])
                    tmp2 = tmp.root.children(3).content // <ul> list
                    RelCharWidth = uman_setRelCharWidth(tmp2, lang)
                end

                text = uman_xml2txt(tmp.root, " ")
                tmp2 = strsplit(text(1), ">")
                if grep(tmp2(1),"Scilab")~=[] then
                    tmp2(1) = "Scilab"
                end
                tmp2 = stripblanks(tmp2)
                tmp2(find(tmp2=="")) = []
                if size(tmp2,"*")>1 & tmp2(1)==tmp2(2)
                    tmp2(2) = []
                end

                text(1) = strcat(tmp2," > ")+ascii(10)
                text(2) = []
                // Removing extra blank lines in deepest sub-blocks
                i = find(text==" ")
                text(i) = []

                // Displaying the summary page +++++++++++++++++++++++
                write(%io(2), text)
                xmlDelete(tmp)
            end
            // targeting the console
        end

    else // The required help file DOES NOT EXIST
         // -------------------------------------
        if grep(options, "w")==[]
            // Local macro:
            // -----------
            if localMacro~=""
                if grep(options,"c")~=[] then
                    clc()
                    mprintf("-> uman %s %s\n\n", fun0, options0)
                end
                head_comments(localMacro)  //
            else // No dedicated page
                 // -----------------
                // Is it a removed feature? We search among removed items:
                res = uman_getRemoved(fun0)
                if res~=[]
                    msg = _("uman", "%%s: ''%%s'' was REMOVED  after Scilab %s")
                    msg = msprintf(msg, res(1,3)) + "\n"
                    if res(1,2)~=""
                        tmp = _("uman", "uman %s  // is a possible replacement")
                        msg = msg + msprintf(tmp, res(1,2)+" "+options0) + "\n";
                    end
                    if strcmp(res(1,3),"5.3")>=0
                        L = ""
                        if LANG~=clang
                            L = "l"
                            if part(LANG,1:2)~="en"
                                L = L + part(LANG,1:2)
                            end
                        end
                        tmp = _("uman", "uman %s w%s   // should work")
                        msg = msg + msprintf(tmp, fun0, L) + "\n"
                    end
                else
                    msg = _("uman"," %s: ''%s'' has no dedicated page.\n")
                end
                mprintf(msg,"uman",fun0)
                if webRef~=[]
                    msg = _("uman", " uman %s w%s  // should work %s.\n")
                    select webRef(1)
                    case "a"
                        tmp = _("uman", "[unloaded ATOMS]")
                    case "fe"
                        tmp = "[FileExchange]"
                    case "o"
                        tmp = _("uman", "[External ressource]")
                    else
                        tmp = ""
                    end
                    mprintf(msg,fun0, strsubst(options0,"g",""), tmp)
                end
                noContent = %t
            end
        end
    end

    if printConsole & grep(options,"j")==[]
        diary([], "resume")   // https://gitlab.com/scilab/scilab/-/issues/12551
    end

    msg1 = ""

    // Using the helpbrowser
    // =====================
    // Must be after print in console => xml_grab() called and got r.parentsummary
    if grep(options,"g")~=[] then
        if helpBrowserWillBeCalled | module==""
            // If the browser is open and its current lang is not LANG,
            // => we close it
          jimport org.scilab.modules.gui.helpbrowser.ScilabHelpBrowser
          hb = ScilabHelpBrowser.getHelpBrowserWithoutCreation()
          if ~isempty(hb)
              if grep(hb.getCurrentURL(), msprintf("scilab_%s_help.jar",LANG))==[]
                  hb.close()
              end
          end
        end
        if helpBrowserWillBeCalled
            // call the help browser in the chosen language
            if grep(options, "s")~=[]
                tmp = r.parentSummary
            elseif htmlName~=""
                tmp = basename(htmlName)
                if grep(tmp, "/.+\-[0-9a-f]{31,32}/","r")
                    tmp = strsubst(tmp, "/\-[0-9a-f]{31,32}/","","r")
                end
            end
            helpbrowser(%helps(:,1), tmp, LANG, %f)

        // fun = raw pattern => litteral search with the help browser
        elseif module==""
            // For "g" and "w" options, the final target must be set
            // -----------------------------------------------------
            //  - if not "x": any equivalence is used
            //  - if "x": the initial fun0 is used
            if grep(options, "x")==[]
                helpbrowser(%helps(:,1), fun, LANG, %f)
                if fun~=fun0
                    msg1 = _("uman", " uman %s %s // used. Run: uman %s x%s // to really target ""%s""\n")
                    msg1 = msprintf(msg1, fun, options0, fun0, options0, fun0)
                end
            else
                helpbrowser(%helps(:,1), fun0, LANG, %f)
                tmp = uman_get_equiv(fun0, options, "scilabFirst") // | "ex2ex" | "exin2in"
                if tmp~=fun0
                    msg1 = _("uman", " You may try: uman %s %s\n")
                    msg1 = msprintf(msg1, tmp, strsubst(options,"x", ""))
                end
            end
        elseif module~="" & (webRef(1)=="a"|webRef(1)=="") // module known but unloaded
            msg1 = _("uman"," [%s] module NOT LOADED => UNAVAILABLE in HELP BROWSER\n")
            msg1 = msprintf(msg1, module)
        end
    end

    // Display of messages
    // ===================
    msg2 = ""
    if printMissingOnline
        msg2 = " ["+module+"] "+..
               _("uman","module EXTERNAL   => ONLINE help page UNAVAILABLE")
    end
    tmp = " "+part("-~", ones(1,max(length([msg1 msg2]))/2).*.[1 2])
    if tmp~="" & (msg1~="" | msg2~="")
        msg0 = " uman " + fun0 + " " + options0 + " :";
        if ~noContent
            write(%io(2),tmp)
            if (msg0~=""), write(%io(2), msg0), end
        end
        if (msg1~=""), write(%io(2), msg1), end
        if (msg2~=""), write(%io(2), msg2), end
        if ~noContent, write(%io(2), tmp), end
    end
endfunction

// ===================================================================

// Copyright (C) 2013 - 2022  Samuel GOUGEON
// Utility function for uman()

function ok = uman_copy_css_files(targetDir)
    // Copies the Scilab CSS files for local HTML pages into the given directory
    // targetDir must already exist (not created here).
    // ok is %T if targetDir has finaly CSS files.
    if ~isdir(targetDir)
        msg = _("uman", "uman_copy_css_files: The directory ''%s'' does not exist.\n")
       error(msprintf(msg, targetDir))
    end
    ok = %t
    if ~isfile(targetDir+"/style.css") then
        F = listfiles(SCI+"/modules/helptools/data/css/*.css");
        for f = F'
           i = copyfile(f, targetDir)
           ok = ok & i~=0
       end
    end
endfunction

// ===================================================================

// Copyright (C) 2013 - 2022  Samuel GOUGEON
// Utility function for uman()

function imgToExtract = uman_process_html_webpage(htmlFile)
    // This function prepares the extracted HTML page in order to be viewed
    //  in the help browser as a local page:
    //  - returns the list of images to be extracted from the JAR file
    //  - converts <img src="jarsci:filename"..> tags into <img src="filename"..>
    //  - passivates <a href="ref" ..> tags into <a href="" ..>
    //
    // htmlFile: path + filename to the HTML file to process
    // imgToExtract: column of images filenames refered to in the page and to
    //  be extracted from the JAR.

    p = htmlRead(htmlFile);

    // Processing the <img src=""> entries
    img = xmlXPath(p, "//img/@src");
    ima = img.content;
    if ima~=[] then
        // Getting the list of images to be extracted:
        i = tokens(strcat(ima,":"),":");
        i = unique(i(i~="jarsci"));
        Scicones = "Scilab"+["Edit" "Execute" "Caution" "Important" "Note" "Tip" "Warning"]+".png";
        imgToExtract = part(setdiff(i, Scicones), 3:$)
        // Cleaning <img src=""> entries = removing the jarsci: prefix
        for j = 1:length(img)
            tmp = strsubst(img(j).content, "jarsci:", "");
            xmlSetAttributes(img(j).parent, ["src" tmp]);
        end
    end
    // Clearing the <a href=".."> => <a href=""> to cancel any navigation:
    href = xmlXPath(p, "//a");
    for i = 1:length(href)
        xmlSetAttributes(href(i), ["href" ""]);
    end
    //
    htmlWrite(p,%t);
    xmlDelete(p);
endfunction

// ===================================================================

// Copyright (C) 2013 - 2022  Samuel GOUGEON
// Utility function for uman()

function localMacro = uman_find_localMacro(fun)
    localMacro = ""
    ierr = execstr("Fun=" + fun + ";", "errcatch")
    if (ierr~=0 | typeof(Fun)~="function")
        if convstr(fun)==fun
            // fun may be the lower-cased name of another function
            n = who("local")
            tmp = find(convstr(n)==fun)
            if tmp~=[]
                for f = tmp
                    execstr("Fun  = "+n(f))
                    if typeof(Fun)=="function"
                        ierr = 0
                        localMacro = n(f)
                        break
                    end
                end
            end
        end
    else
        localMacro = fun
    end
endfunction

// ===================================================================

// Copyright (C) 2013 - 2022  Samuel GOUGEON
// Utility function for uman()

function [fun, options] = uman_updateQueryFromEquiv(fun, options)
    // Some equivalences have "£" inserted: prefixing chars are options to force
    //  By default, "s" is forced, meaning that the equivalence is designed
    //  to target a parent summary:
    if grep(fun,"£")~=[] then
        s = strsplit(fun, "£", 1)
        fun = s(2)
        if s(1)==""
            if grep(options, "s")==[]
                options = "s" + options
            end
        else
            options = s(1) + options
        end
        // Update priorities:
        if grep(options, "s")~=[]
            options = uman_strsubst(options, ["w" "a" "b" "d" "e" "h" "p"],"")
        end
        if grep(options, "u")~=[]
            options = uman_strsubst(options, ["w" "s" "a" "b" "d" "e" "h" "p"],"")
        end
    end
endfunction

// ===================================================================

// Copyright (C) 2013 - 2022  Samuel GOUGEON
// Utility function for uman()

// This function looks in all registered JHelpMap files for the entry with
//  target="fun". If it is found, the path to the related HTML file is
//  returned, as well as the name of the related module ("scilab" or external)
function [htmlName, jhmLine] = uman_findItemInJHelpMaps(fun, jhmTab, mapDir, grepCase)
    htmlName = ""
    jhmLine = []
    for jhm = jhmTab'
        //doc = xmlRead(jhm)
        //e = xmlXPath(doc,"//mapID[@target="""+fun+"""]")
       doc = mgetl(mapDir+jhm(2))
       i = grep(doc, "/target="""+fun+"""/"+grepCase, "r")
       if length(i)>1
           i = grep(doc, "/target="""+fun+"""/", "r")
       end
       if i~=[] // We found the pattern!
           // Name of the .html file to be extracted?
           search = "/^.*?target="""+fun+""" url=""(.+?.html).*?"".*$/"+grepCase
           htmlName = uman_strsubst(doc(i), search, "$1","r")
           jhmLine = jhm
           break
       end
    end
endfunction

// ===================================================================

// Copyright (C) 2013 - 2022  Samuel GOUGEON
// Utility function for uman()

function uman_disp(man_contents, options)
    s = man_contents;
    // If the page is an Xcos palette one => force the "a" option
    if s.isXcosPalette & ..
        (options=="" | grep(options, ["s" "a" "p" "d" "e" "h"])==[]) then
        options = options + "pdeh"
    end
    // Finally displays the formatted help text content into the console
    if or(part(LANG,[1 2])==["ja" "zh"])
        lengthFactor = 1.5
    else
        lengthFactor = 1
    end
    u = "----------------------------------------------------------" + ..
        "----------------------------------------------------------"
    c = %io(2)

    t = s.path
    if grep(options,"u")==[]
        write(c, [ " " ; t ; part(".",ones(1,length(t))) ; " "])
        write(c, strcat(justify([s.refname s.refpurpose],"l")," - ","c"))
    else
        write(c, [ " " ; t ; part(".",ones(1,length(t)))])
    end

    if s.synopsis~=[] then
        t = s.synopsisTitle
        if grep(options, "u")==[]
            write(c, [ " " ; t ; part(u,1:length(t)*lengthFactor) ; " "+s.synopsis ])
        else
            write(c, [ convstr(t,"u") ; " "+s.synopsis ])
        end
    end

    if grep(options,"u")==[]
        sections = man_contents.sections
        ns = size(sections,"*")

        // Is there any parameters section?
        // If no, it will be possible to force the display of description ones
        has_params_section = %f // if false, description sections are displayed
        for is = 1:ns
            s = sections(is)
            has_params_section = has_params_section | (s.type=="parameters" & s.contents~=[])
        end

        // Loop over other sections, in the order they appear in the original page
        for is = 1:ns
            s = sections(is)
            tmp = s.type
            if    tmp=="see_also" | ..        // always displayed
                ((tmp=="description" & ..
                     (grep(options,"d")~=[] | ..
                     (grep(options,"p")~=[] & ~has_params_section & ~man_contents.isXcosPalette))) | ..
                (tmp=="contents"    & grep(options,"d")~=[]) | ..
                (tmp=="parameters"  & grep(options,"p")~=[]) | ..
                (tmp=="examples"    & grep(options,"e")~=[]) | ..
                (tmp=="history"     & grep(options,"h")~=[]))
                t = s.title
                if t~=""
                    write(c, [ " " ; t ; part(u,1:length(t)*lengthFactor)])
                    //write(c, [ "" ; t])
                end
                if s.contents~=[]
                    if t==""
                        write(c,[" ";" "])
                    end
                    if tmp=="see_also"
                        s.contents = part(s.contents, 3:$)  // trimming starting stars
                        global umanSeeAlsoOnlyKeywords umanMaxLinesWidth
                        if umanSeeAlsoOnlyKeywords
                            e = [];
                            i = grep(s.contents,"/^ [^\s].*?(—|$)/","r");
                            tmp = "/^ ([^\s].*?)(?: —.*$|$)/"
                            for j = i
                                [?,?,?,m] = regexp(s.contents(j),tmp);
                                e = [e m];
                            end
                            e = strcat(stripblanks(e)," | ");
                            tmp2 = umanMaxLinesWidth
                            tmp = lines()
                            if tmp2~=[]
                                tmp2 = max(min(tmp2,tmp(1),110),50)
                            else
                                tmp2 = max(min(tmp(1),110),50)
                            end
                            e = uman_wraplines(e, tmp2-2)
                            s.contents = e
                        end
                    end
                    write(c, " "+strsubst(s.contents,"¤", " "))
                end
            end
        end
    end
endfunction

// ===================================================================

// Copyright (C)   2013 - 2022    Samuel GOUGEON
// Utility function for uman()

function r = uman_parse_html2sections(htmlf, fun, LANG, options)
    // htmlf : HTML file name
    // fun : function's name or item
    // LANG: ##_## language string
    // options: options string from uman()
    // r. : structure with fields
    //  .fun .title  .path .refname .refpurpose .synopsisTitle .synopsis
    //  .paramTitle  .parameters .examples .seeAlsoTitle .seeAlso
    //  .parentSummary .isXcosPalette

    lang = part(LANG,1:2)

    html = mgetl(htmlf)
    search  = ["<br>" "<meta .*?>" "<[^a-zA-Z!\/]" "&eacute;" "&egrave;"];
    replace = ["<br/>"    ""           "&lt;"          "é"        "è"   ];
//    search  = [search  "&&"         "&([^a-z#])" ];
//    replace = [replace "&amp;&amp;" "⛔$1"       ];
    search  = [search  "&&"         ];
    replace = [replace "&amp;&amp;" ];
    search =  "/" + search + "/"
    html2 = uman_strsubst(strcat(html, " "), search, replace, "r");
    html2 = strsubst(html2, "/&(?!([a-z]+?;)|(#[0-9]{2,}+;))/", "⛔", "r");

    // Support to <sup> and <sub> tags
    search  = [ "/<sup>"+["(.?)" "(.*?)"]+"</sup>/" ..
                "/<sub>"+["(.?)" "(.*?)"]+"</sub>/"
               ]
    replace = ["\^$1" "\^{$1}" "_$1" "_{$1}"]
    html2 = uman_strsubst(html2, search(:), replace, "r")

    // URLs GET's & separators must be replaced with %26 in order to not break
    //  XML parsing. Replacing all cannot be done with TCL uman_strsubst because
    //  "&" has a special meaning in this TCL regsub context
    // => using a while loop here:
    src = ""
    while or(html2~=src) do
        src = html2
        html2 = uman_strsubst(src, ["|(href=""http:[^\""]*?)&([^\""]*?"")|" "|& |"],..
                                   ["$1%26$2" "&amp; "], "r")
    end

    doc = xmlReadStr(html2)

    // If the language is ja_JP: with must assess the fraction of content in
    // japanese vs latin, in order to tune the average character width used
    // to wrap long lines
    if or(lang==["ja" "zh"])
        tmp = doc.root.children(2).content  // Literal content of the body
        RelCharWidth = uman_setRelCharWidth(tmp, lang)
    end

    // Table of <pre.. </pre> sections (badly processed as XML)
    [s,e,tmp,preTable] = regexp(strcat(html,"££"),"%<pre.*?>(.*?)</pre>%mi")
    e = xmlXPath(doc,"//pre")
    for i = 1:size(s,"*")
        tmp = preTable(i)
        preTable(i) = uman_cleantxt(preTable(i),"tags|htmlent|style")
        // now we forge an id = full content cleaned
        preTableNoTag(i) = uman_strsubst(e(i).content, [" " "££" "\⛔"], ["" "" "&"])
    end

    htmlcat = strcat(html, "££")

    r.fun = fun

    // Title
    e = xmlXPath(doc,"//title")
    r.title = e.content

    // Path in help
    // ------------
    e = xmlXPath(doc,"//span[@class=""path""]")
    tmp = strsplit(e.content, ">")
    if grep(tmp(1), "Scilab")~=[] then
        tmp(1) = "Scilab"
    end
    tmp = stripblanks(tmp)
    tmp(find(tmp=="")) = []
    if tmp(1)==tmp(2)
        tmp(2) = []
    end
    r.path = strcat(tmp," > ")

    // Is it an Xcos page? (=> will filter out some specific sections)
    Xcos = grep(r.path, " Xcos ")~=[];
    r.isXcosPalette = grep(r.path, " Xcos > palettes > ")~=[];

    // Parent Summary (from path)
    tmp = "%<span class=""path"".*?>.*?</span>%mi"
    [s, e, tmp] = regexp(html2, tmp)
    tmp2 = "%href="".*?""%mi"
    [s, e, tmp2] = regexp(tmp, tmp2)
    r.parentSummary = part(tmp2($), 7:$-6)

    // refname(s)
    e = xmlXPath(doc,"//h1[@class=""refname""]")
    r.refname = xmlAsText(e)'

    // refpurpose
    e = xmlXPath(doc,"//p[@class=""refpurpose""]")
    r.refpurpose = uman_cleantxt(xmlAsText(e)', "blanks")
        // Some pages presenting multiple equivalent functions do not repeat
        // the short description for last aliases
        // (printf.. https://gitlab.com/scilab/scilab/-/issues/13748 )
        // We must complete them here:
    tmp =  r.refpurpose
    if size(tmp,1)<size(r.refname,1) & r.refpurpose($)~=[] then
        r.refpurpose = [tmp ; emptystr(size(r.refname,1)-size(tmp,1),1)+r.refpurpose($)]
    end

    // synopsis title
    CS = struct("en","Syntaxes",..
                "fr","Usages",..
                "ja","呼出し手順", ..
                "pt","Seqüência de Chamamento",..
                "de","Aufruf", ..
                "ru","Последовательность вызова",..
                "zh","调用序列",..
                "fa","دنباله‌ی فراخوانی")
    if isfield(CS,lang) then
        r.synopsisTitle = CS(lang)
    else
        e = xmlXPath(doc,"//div[@class=""refsynopsisdiv""]")
        if e.size>0 then
            r.synopsisTitle = e(1).children(1).content
        else
            r.synopsisTitle = _("uman", "Syntax")
        end
    end

    // Synopsis
    //s = xmlXPath(doc,"//div[@class=""synopsis""]")
    [s,e,M] = regexp(htmlcat, "%<div class=""synopsis""><pre>(.*?)</pre>%")
    tmp = []
    for m = M'
        s = strsplit(uman_cleantxt(m,"tags|htmlent|style"),"££")
        tmp = [tmp ; s ]
        if size(s,1)>1 then
            tmp = [tmp ; ""]
        end
    end
    tmp = strsubst(tmp, "&amp; ", "& ")     // (spaces matter..)
    if tmp($)=="" then // removing the last blank line
        tmp($) = []
    end
    if size(r.refname,1)==1 then    // Removing blank lines
        // tmp(find(tmp=="")) = []
    end
    // When a page has several xml:id, get syntaxes matching only the pattern.
    // Exceptions:
    //  * Lines beginning with ".." are the continuation of previous ones (then
    //    assumed to be always relevant)
    //  * control blocks (if, select, etc)
    if tmp~=[] & and(fun<>getscilabkeywords()(2)) then
        i = grep(" " + tmp, "/(^ $|[^a-zA-Z]" + fun + "|^\s*\.\.)/","r")
        if i~=[] & stripblanks(strcat(tmp(i)))<>"" then
            tmp = tmp(i)
        end
    end

    // Synopses post-processing: alignment (if required)
    global umanAlignSyntaxes
    if umanAlignSyntaxes==[] then
        umanAlignSyntaxes = "r"     // default alignment mode
    end
    if size(tmp,1)>1 & or(umanAlignSyntaxes==["l" "r"]) then
        // Aligning  "= fun(..)"
        q  = strstr(tmp, "= "+fun+"(");
        Lq = length(q);
        k  = find(Lq>0);
        hk = ""
        if k~=[] then
            hk = justify(strncpy(tmp(k), strcspn(tmp(k), "=")), umanAlignSyntaxes);
            tmp(k) = hk + " " + q(k)
        end
        // Aligning remaining "fun"
        k2 = find(Lq==0);
        if k2~=[] then
            hwidth = length(hk(1));
            s = [];
            for i = 1:length(k2)
                istart = regexp(tmp(k2(i)), "|"+fun+"|");
                if istart>1 then
                    s = [s ; strsplit(tmp(k2(i)), istart-1)'];
                else
                    s = [s ; ["" tmp(k2(i))]]
                end
            end
            if k~=[]
                s = [s ; [part(".",ones(1,hwidth+3)) ""]];
                s(:,1) = justify(s(:,1), umanAlignSyntaxes)
                s($,:) = []
            else
                s(:,1) = justify(s(:,1), umanAlignSyntaxes)
            end
            tmp(k2) = strcat(s, "", "c")
        end
    end
    // removing left justification for ".." continuation lines
    k = grep(tmp,"/^\s*\.\./","r")
    if k~=[] then
        tmp(k) = "  " + stripblanks(tmp(k))
    end
    //
    r.synopsis = strsubst(tmp, "& &", "&&");

    if grep(options, "u")==[]

        // REFSECTIONS
        // ===========
        refsections = xmlXPath(doc,"//div[@class=""refsection"" or @class=""refsect1""]")
        nrs = length(refsections)

        // MAIN LOOP over SECTIONS of all TYPES
        // ------------------------------------
        i = 0;
        for j = 1:nrs
            i = i+1;
            rs = refsections(j)
            secTitle = xmlXPath(rs, "./h3[@class=""title""]")
            if size(secTitle,"*")>0
                tmp = uman_cleantxt(secTitle(1).content, "blanks")
                [secType, typeName] = uman_typeof_section(tmp, lang)
                if Xcos & or(typeName==["Contents" "Contenu" "Module" "Palette" ..
                                 "Block Screenshot" "Aperçu"]) // To skip
                    i = i-1
                    continue
                end
                r.sections(i).title = typeName
                contents = uman_xml2txt(rs, "")
                contents(1) = []     // suppressing the embedded title
            else
                secType = "description"
                r.sections(i).title = ""
                contents = uman_xml2txt(rs, "")
            end
            r.sections(i).type  = secType
            if secType=="examples" then
                contents = uman_strsubst(contents, "/\[IMAGE\: .*?\]/","","r")
                contents = strsubst(contents, "\&", "&")
            elseif secType=="see_also"
                k = find(stripblanks(contents)=="")
                contents(k) = []
            end
            while contents(1)=="" | grep(contents(1),"/^\s*$/","r")~=[]
                contents(1) = []
            end
            while contents($)=="" | grep(contents($),"/^\s*$/","r")~=[]
                contents($) = []
            end

            r.sections(i).contents = contents
        end
    end

    xmlDelete(doc)
endfunction

// ===================================================================
// Copyright (C)  2013 - 2022   Samuel GOUGEON
// Utility function for uman()

function [options, LANG] = uman_splitOptionsLang(options)
    i = strindex(options, "l")
    LANG = ""
    if i~=[] then
        LANG    = part(options, (i(1)+1):(i(1)+2))
        if LANG==""
            LANG = "en"
        end
        options = uman_strsubst(options, ["/l.._../i" "/l../i"], ["" ""], "r")
    end
endfunction

// ===================================================================
// Copyright (C)  2013 - 2022   Samuel GOUGEON
// Utility function for uman()

// Japanese and Chinese characters are wider than latin ones.
// This function estimates the average char relative width according to the
//  estimated amount of Japanese or Chinese characters in the passed nakedText,
//  over all characters (mixed with latin ones).
// This relative width is needed when wrapping lines.
function relCharWidth = uman_setRelCharWidth(nakedText, lang)
    tmp = uman_cleantxt(nakedText, "blanks")  // removal of consecutive blanks
    Nchar = length(tmp)
    Nascii = length(ascii(tmp))
    if or(lang==["ja" "zh"])
        // 100% japanese => Nascii~3*Nchar  <=> 1 jaChar ~ 1.4 latinChar
        if Nascii/Nchar>1.5     // detection threshold
            relCharWidth  = 1.4
        else
            relCharWidth  = 1
        end
    end
endfunction

// ===================================================================

// Copyright (C)  2013 - 2022   Samuel GOUGEON
// Utility function for uman()

function [secType, typeName] = uman_typeof_section(secTitle, lang)
    // id vs titles for sections
    langs = [ "en" "de" "fr" "ja" "pt" "ru" "zh" "fa"]
    ID = ["parameters"              "description"  "examples" ]
    en = ["Parameters,Parameter,arguments"    "Description"  "Examples,example"]
    de = ["Parameter"               "Beschreibung" "Beispielen,beispiel,beispiele"]
    fr = ["Paramètres"              "Description"  "Exemples,exemple"]
    ja = ["設定,引数,パラメータ"       "説明"         "例"]
    pt = ["Parâmetros,Argumentos"   "Descrição"    "Exemplos,exemplo"]
    ru = ["Аргументы,Параметры"     "Описание"     "Примеры,Примерами,ример,примеров"]
    zh = ["設置"                    "描述"          "示例,例子"]
    fa = ["آرگومان‌ها"               "توصیف"        "مثال‌ها"]

    ID = [ ID "see_also"    "history"     "contents"        ]
    en = [ en "See Also"    "History"     "Contents,Summary"]
    de = [ de "Siehe auch"  "Historische" "Zusammenfassung,Summarisch,Inhalt"]
    fr = [ fr "Voir aussi"  "Historique"  "Sommaire,Contenu"]
    ja = [ ja "参照"         "履歴"        "内容,まとめ"       ]
    pt = [ pt "Ver também"  "Histórico"   "Conteúdos,Resumo"]
    ru = [ ru "Смотрите также" "История"  "Содержание, Резюме"]
    zh = [ zh "参见"         "历史的"       "内容,摘要"        ]
    fa = [ fa "همچنین ببینید" "تاریخچه"   "فهرست,خلاصه"     ]
        // "See also" in Farsi (bug to be fixed)

    k = find(langs==lang)
    if k==[] then
        lang = "en"
    end
    execstr("Ref = "+lang)
    secType = "description" // default
    typeName = secTitle     // default
    L = length(secTitle)
    secTitle= convstr(secTitle)

    ref = convstr(Ref)
    found = %f
    while ~found
        for i=1:size(ref, 2)
            tmp  = tokens(ref(i), ",")
            tmp2 = tokens(Ref(i), ",")
            [j,k] = grep(secTitle, tmp)
            if j~=[] & (i~=1 | or((L-length(tmp(k)))<3)) then
                secType = ID(i)
                if or((L-length(tmp(k)))<3)
                    typeName = tmp2(1)
                end
                found = %t
                break
            end
        end
        tmp = convstr(en)
        if found | and(ref==tmp), break, end
        ref = tmp
    end
endfunction

// -------------------------------------------------------------------

// Copyright (C)  2013 - 2022   Samuel GOUGEON
// Utility function for uman()
function out = uman_cleantxt(txt, cleanjobs)
    // cleanjobs: "tags|htmlent|catlines|style|blanks"
    cleanjobs = tokens(cleanjobs,"|")'
    out = txt
    for cleanjob = cleanjobs
        select cleanjob
        case "tags"
             out = uman_strsubst(out, "/<.*?>/", "", "r")
         case "catlines"
             for i = 1:size(out,"*")
                out(i) = strcat(stripblanks(strsplit(out(i),"££"))," ")
             end
         case "htmlent"
             patterns = ["&#0034;" "&#0038;" "&#0039;" "&#0060;" "&#0062;" "&gt;" "&lt;" "&quot;" ]
             replaces = [ """"       "\&"       "''"      "<"       ">"       ">"   "<"   """"    ]
            out =  uman_strsubst(out, patterns , replaces)
         case "style"
            out =  uman_strsubst(out, "/([^ \=<>~])=([^ \=''""])/", "$1 = $2", "r")
            out =  uman_strsubst(out, "/([^\s]);([^\s])/", "$1; $2", "r")
        case "blanks"
            out = uman_strsubst(out, "/((\s{2,}|\t))/", " ", "r")
        end
    end
endfunction

// -------------------------------------------------------------------

// Copyright (C)  2013 - 2022   Samuel GOUGEON
// Utility function for uman()

function out = uman_wraplines(text, ncol)
    if isdef("isInCell") & isInCell then
        out = text
        return
    end
    // LANG must be set in the calling environnement
    if text==[] then
        out = []
        return
    end
    if or(LANG==["ja_JP" "zh_CN"]) then
        // RelCharWidth is defined in uman_parse_html2sections() in the calling environnement
        nchar = floor(ncol/RelCharWidth)  // because symbols are wider than latin ones
    else
        nchar = ncol
    end
    nchar = nchar-1
    text = stripblanks(text)
    Sep = " "
    if LANG=="ja_JP" then
        Sep = [Sep "。" "・"]
//        [T, sep] = strsplit(text, Sep)
//        sep($+1) = ""
//    else
//        if text~=[]
//            T = tokens(text," ")    // 30x faster than strsplit...
//            sep = emptystr(T)+" "
//        else
//            T = []
//            sep = ""
//        end
    end
    [T, sep] = strsplit(text, Sep)
    sep($+1) = ""
    LT = length(T)+1
    out = []
    line = ""   // current line
    lineL = 0   // lineLength
    for i = 1:size(T,"*")
         if lineL+LT(i)<=nchar | line==""
             line = line + T(i) + sep(i)
             lineL = lineL + LT(i)
         else
             if lineL+LT(i)>nchar & LANG=="ja_JP" & grep(line,"/(—\s|[^。・\s])$/","r")~=[]
                 line = line + T(i) + sep(i)
                 out = [out ; line]
                 line = ""
                 lineL = 0
             else
                 out = [out ; line]
                 line = T(i) + sep(i)
                 lineL = LT(i)
             end
         end
    end
    if stripblanks(line,%t)~="" then
        out = [out ; line]
    end
    out = stripblanks(strsubst(out, "¤"," "))
endfunction

// -------------------------------------------------------------------

// Copyright (C)  2013 - 2022   Samuel GOUGEON
// Utility function for uman()

function [newT, cw] = uman_wraptable(T, nchar)
// * Multicolumn table with long unwrapped lines (ex: uman atomsSetConfig ac )
//   . nchar = largeur totale du tableau, en nombre de caractères
//   . newT : nouveau tableau après césurage
//   . cw : vecteur ligne des largeurs de colonnes
//   . nc = nombre total de colonnes
//   . C1 = colonnes dont la largeur est < nchar/nc+2
//     n1 = nombre de colonnes C1
//     w1 = largeur cumulée des colonnes étroites C1
//     C2 = colonnes dont la largeur est >= nchar/nc+2
//     n2 = nombre de colonnes C2
//     C3 = colonnes parmi les C2 dont la largeur est < (nchar-w1)/n2
//     n3 = nombre de colonnes C3
//     w3 = largeur cumulée des colonnes demi-larges C3
//     C4 = colonnes parmi les C2 dont la largeur est >= (nchar-w1)/n2
//          = colonnes à wrapper.
//     n4 = nombre de colonnes C4
//          largeur de wrappage wW = (nchar-w1-w3)/n4 - 2
    nc = size(T,2)
    T = stripblanks(T);

    // To compute the columns widths, we must not take into account rows that
    //  have colspan cells (cell starting with "#§" where # is the colspan range)
    // Rows having at least one colspan:
    colspanLines = grep(strcat(T,"","c"), "/^\d+§/", "r")
    tmp0 = T
    tmp0(colspanLines,:) = []

    // Column widths: if there are some "€", at least one cell has a serialized
    //  content that must be expanded / unserialized before getting the true
    //  content width
    cw = []
    for j = 1:nc
        c = tmp0(:,j)
        if grep(c, "€")~=[] then
            tmp = []
            for i = 1:size(c,1)
                tmp = [tmp ; tokens(c(i), "€")]
            end
        else
            tmp = c
        end
        cw(1,j) = max(length(tmp))
    end
    // Actual total witdh without cellspacings:
    contentsWidth = nchar - nc*(1+withUmanBorders) - 2
    // Computing columns widths and toBeWrapped status:
    C1 = find(cw < ceil(contentsWidth/nc))
    n1 = length(C1)
    w1 = sum(cw(C1))
    C2 = find(cw >= ceil(contentsWidth/nc))
    n2 = length(C2)
    if n2>0 then
        C3 = find(cw(C2) < ceil((contentsWidth-w1)/n2))
        C3 = C2(C3)
        C4 = find(cw(C2) >= ceil((contentsWidth-w1)/n2))
        C4 = C2(C4)
    else
        C3 = []
        C4 = []
    end
    n3 = length(C3)
    w3 = sum(cw(C3))
    n4 = length(C4)

    if n4>0 then    // Some wide columns to be wrapped exist
        wW = ceil((contentsWidth-w1-w3)/n4)+4
    end

    if n4==0 & grep(T, "€")==[]
        newT = T
    else
        newT = []
        // Loop over the rows of the table
        for i = 1:size(T,1)
            c = list()
            // Loop over the columns of the row
            for j = 1:size(T,2)
                if or(j==C4) & grep(T(i,j), "/^\d+§/", "r")==[]
                    tmp2 = tokens(T(i,j), "€")   // unserializes the cell content
                    tmp = []                 // column of rows inside the table cell
                    for i2 = 1:size(tmp2,1)
                        tmp3 = uman_wraplines(tmp2(i2), wW)
                        if size(tmp3,1)==1 & length(tmp3)>wW // wide unbreakable bloc
                            tmp3 = uman_wraplines(strsubst(tmp2(i2),"¤"," "), wW)
                        end
                        if size(tmp3,1)>1
                            if grep(tmp2(i2),"/^\s*\*/", "r")~=[]
                                indent = part("¤", 1:strspn(tmp2(i2), " *")-1)
                            elseif grep(tmp2(i2),"/^\s*\[0-9]*\./", "r")~=[]
                                indent = part("¤", 1:strspn(tmp2(i2), " *0123456789.")-1)
                            else
                                indent = part("¤", 1:strspn(tmp2(i2), " ")-1)
                            end
                            tmp3(2:$) = indent + tmp3(2:$)
                        end
                        tmp = [tmp ; tmp3]
                    end
                    c(j) = tmp
                    nL(j) = size(tmp,1)
                else
                    c(j) = tokens(T(i,j),"€")
                    nL(j) = size(c(j),1)
                end
            end
            nnL = max(nL)
            BoL = []    // new Block of (wrapped) Lines
            for j = 1:size(T,2)
                BoL = [BoL [c(j) ; emptystr(nnL-nL(j),1)]]
            end
            newT = [newT ; BoL]
        end
    end
endfunction

// -------------------------------------------------------------------

// Copyright (C)  2013 - 2022   Samuel GOUGEON
// Utility function for uman()

function [txt, inline] = uman_xml2txt(node, lineHead)
    // Upstream variables used: options, secType, preTableNoTag, preTable
    //
    // Pieces of HTML code (template to decode):
    //<code..>bla bla</code>   : =>  [bla bla]  // cancelled
    //<a href=""..>bla bla</a> : => _{bla bla}  // cancelled
    // <ul class="itemizedlist">
    //       <li><p class="para">..</p> : => nouvelle ligne à chaque <p..>
    //           <p..>...</p>
    //       </li>
    // </ul>
    // <img ..> among which
    //   <img src="ScilabNote.png"/> and <img src="ScilabWarning.png"/>
    //   <img src="ScilabTip.png"/> and <img src="ScilabCaution.png"/>
    //   <img src="ScilabImportant.png"/>
    //
    // "¤" is used as non-breakable space.

    RelCharWidth = RelCharWidth // defined in the calling environnement
    inline = %F
    if ~isdef("isInCell")
        isInCell = %f
    end
    thistxt = []
    thisline = []
    name = convstr(node.name)
    nc = node.children.size     // number of children

    // Max line width limited to umanMaxLinesWidth or by default to 90 chars
    cw = lines()
    tmp = 90
    global umanMaxLinesWidth
    tmp2 = umanMaxLinesWidth
    if tmp2~=[]
        if type(tmp2)~=1 | ~isreal(tmp2) | tmp2<50
            tmp = 90
        else
            tmp = tmp2
        end
    end
    cw = min([cw(1) tmp])
    // ---------------------------------------------------------------
    if name=="code" then
        // "¤" is used as non-breakable space (back-substituted later):
        txt = strsubst(uman_cleantxt(node.content, "style|blanks"), " ", "¤")
        // https://gitlab.com/scilab/scilab/-/issues/15335
        inline = %T

    elseif name=="a" then
        href = node.attributes.href;
        txt = uman_cleantxt(node.content, "blanks");
        if txt~="" then
            global umanPrintHyperlinks
            ulinkInSeealso = %f
            if isdef("secType")
                ulinkInSeealso = secType=="see_also" & node.attributes.class=="ulink"
            end
            if (ulinkInSeealso | ..
               (umanPrintHyperlinks~="content" & ..
                    (~isdef("secType") | secType~="see_also") & ..
                    grep(options, "s")==[] .. // Links canceled in summaries
                )) & (~isdef("secType") | secType~="contents")
                if umanPrintHyperlinks=="[content]" & ~ulinkInSeealso
                    txt = "[" + txt + "]"
                else // content [url]
                    if grep(href,"://")~=[]
                        if href~=txt then
                            txt = txt + " [" + href + "]";
                        else
                            if ulinkInSeealso
                                txt = href
                            else
                                txt = "[" + href + "]";
                            end
                        end
                    elseif basename(href)~=txt
                        txt = txt + " [" + basename(href) + "]";
                    else
                        txt = "[" + txt + "]"
                    end
                end
            end
        end
        inline = %T

    elseif or(name==["text" "i"]) then
        txt = uman_cleantxt(node.content, "blanks")
        inline = %T

    elseif or(name==["h3" "h4" "h5"]) then
        tmp = uman_cleantxt(node.content, "blanks")
        if name=="h4" then
            txt = [""; tmp ; part("-·", ones(1,ceil(length(tmp)*RelCharWidth/2)).*.(1:2))]
        elseif name=="h5" then
            txt = [""; tmp ; part("·", ones(1,length(tmp)*RelCharWidth))]
        else
            txt = tmp;
        end
        txt = " "+txt;

    elseif or(name==["h5"]) then
        //txt = convstr(uman_cleantxt(node.content, "blanks"), "u")
        txt = uman_cleantxt(node.content, "blanks")
//        u = "."
//        if name=="h3" then
//            u = "-"
//        end
//        txt = [txt ; part(u, ones(1,length(txt)*RelCharWidth))]

    // ---------------------------------------------------------------
    elseif or(name==["b" "em"]) then
        txt = uman_cleantxt(node.content,"blanks")
        global umanAsciiStyling
        if umanAsciiStyling then
            if  name=="em" then
                txt = "/"+txt+"/"
            else
                txt = convstr(txt,"u")
            end
        end
        inline = %T

    // ---------------------------------------------------------------
    elseif name=="varname"
        txt = "[" + uman_cleantxt(node.content, "blanks") + "]"
        inline = %T

    // ---------------------------------------------------------------
    //    elseif name=="a"
    //        // <a> over complex content will ignore content's format :
    //        //txt = "_{" + node.content + "}"
    //        txt = node.content
    //        inline = %T

    // ---------------------------------------------------------------
    elseif or(name==["div" "p" "span" "li" "dd"])
        if name=="div" & node.attributes.class=="programlisting"
            k = find(preTableNoTag==strsubst(node.content," ",""))
            if k~=[]
                //tmp = strsplit(preTable(k(1)),"££")
                //tmp = part(".", ones(1,40)) // "......................"
                //txt = [ tmp; strsplit(preTable(k(1)),"££") ; tmp ; ""]
                txt = [ strsplit(preTable(k(1)),"££") ; ""]
            else
                txt = ""
            end

        elseif name=="div" & node.attributes.class=="screen"
            txt = ""
            k = find(preTableNoTag==strsubst(node.content," ",""))
            if k~=[]
                tmp = strsplit(preTable(k(1)), "££")
                if tmp(1)=="", tmp(1)=[], end
                if tmp($)=="£", tmp($)=[], end
                if grep(tmp($),"/^\s*$/","r")~=[], tmp($)=[], end
                tmp2 = max(length(tmp))
                if tmp~=[]
                    tmp = ":  "+part(tmp,1:tmp2)+" :"
                end
                tmp2 = part(".",ones(1,tmp2+5))
                txt = [ tmp2; tmp; tmp2 ]
            end
        else
            // Wrapping width
            // wW = cw-length(lineHead)-3
            wW = cw - length(lineHead)
            if wW<30, wW = 30, end            // minimal width = 30 chars
            //
            tmp = %f
            if isdef("secType"), tmp = secType<>"history", end
            forceInLine = (name=="li" & (isdef("simpleContent", "n") | (isInCell & tmp)))
            if name=="li" & forceInLine
                tmp = xmlXPath(node, "./ul")  // are there any nested sub-list?
                if size(tmp, "*")>0         //   yes:
                    forceInLine = %f
                end
            end

            for i = 1:nc
                [ tmp, inL ] =  uman_xml2txt(node.children(i), lineHead)

                if inL | forceInLine then
                    if thisline==[]
                        thisline = strcat(tmp, " ")
                    elseif tmp~=[]
                        thisline = thisline + strcat(tmp, " ")
                    end
                else
                    tmp2 = uman_cleantxt(thisline,"blanks")
                    tmp2 = uman_wraplines(tmp2, wW)
                    thistxt = [ thistxt ; tmp2 ; tmp ]
                    thisline = []
                end
            end
            if thisline~=[]
                tmp2 = uman_cleantxt(thisline,"blanks")
                tmp2 = uman_wraplines(tmp2, wW)
                txt = [ thistxt ; tmp2 ]
            else
                txt = thistxt
            end
            if name=="span" & size(txt,1)==1 then
                inline = %T
            end
        end

    // ---------------------------------------------------------------
    elseif name=="pre"
        // preTable, preTableNoTag = external variables
        k = find(preTableNoTag==strsubst(node.content, " ", ""))
        txt = strsplit(preTable(k(1)), "££")

    // ---------------------------------------------------------------
    elseif name=="img"  // ex: <img src='jarsci:./PlotSparse_1.png'/>
        // works with Scilab pages. Untested with external packages
        txt = node.attributes.alt
        tmp = node.attributes.src;
        tmp = tokens(tmp,"/")
        inline = %t
        if grep(tmp($),"_LaTeX_")<>[] & txt<>[] & stripblanks(txt)<>"" then
            //txt = strsubst(tmp2,"\ ","¤")
            if grep(txt, "\n") <> []
                txt = strsplit(txt, "\n")
                inline = %F
            end
            txt = strsubst(txt,"\ ", ascii([10]))
            txt = strsubst(txt," ","¤")
            txt = strsubst(txt,"\exp\","exp\")
            txt = strsubst(txt,"\exp(","exp(")
            txt = strsubst(txt,"{\infty}","∞")
            txt = strsubst(txt,"\infty","∞")
            tmp3 = tokens("alpha,beta,gamma,delta,epsilon,dzeta,eta,theta,iota,kappa,lambda,mu,nu,xhi,omicron,pi,rho,sigma,tau,upsilon,phi,xhi,psi,omega", ",");
            tmp4 = strsplit("αβγδεζηθικλμνξοπρστυφχψω")
            txt = uman_strsubst(txt, "\"+tmp3, tmp4)
            txt = strsubst(txt,"\sqrt","√")
            txt = strsubst(txt,"\int_","∫_")
            txt = strsubst(txt,"\int¤","∫¤")
            txt = strsubst(txt,"\left(","(")
            txt = strsubst(txt,"\right)",")")
            txt = strsubst(txt,"\begin{eqnarray}","")
            txt = strsubst(txt,"\end{eqnarray}","")
            txt = uman_strsubst(txt, "/([_^])\{(.)\}/", "$1$2", "r")
            txt = uman_strsubst(txt, "/\\frac\{(.)\}\{(.)\}/", "$1/$2", "r")
            //txt = uman_strsubst(txt, "/\\frac\{(.{2,})?\}\{(.)\}/", "($1)/$2", "r")
            txt = uman_strsubst(txt, "/\\frac\{(.)\}(\{.{2,}?\})/", "$1/$2", "r")
        else
            txt = "[IMAGE:¤"+tmp($)+"]"
        end

    // ---------------------------------------------------------------
    elseif name=="dl"     // definitions list
        // <dl> : definitions list : ignorer content ; traiter les children
        //   <dt><span class="term">..</span>      : definition term
        //  OR   <a name="<term_id>"></a><span class="term">..</span>
        //       <dd><p class="para">...</p></dd>  : definition
        //   </dt>
        // </dl>
        // Processing all <dt> entries
        terms = []     // matrix of terms
        defs  = list() // list of related definitions
        i = 1    // n° of <dl>'s children
        it = 0   // n° of terms
        maxL = 8   // maximal length of "alignable" terms
        minL = 5    // minimal indentation of lines for the bloc
        while i<=nc
            // only <dt>..</dt> should be considered:
            // WARNING: IF THERE IS SOMETHING BEFORE <TERM>, IT IS IGNORED/SKIPPED!
            // TERM:
            while node.children(i).name~="dt" & i<nc
                i = i+1
            end
            dt = node.children(i)
            // Searching for the <span class="term">..</span> tag:
            j = 1
            sp = dt.children(1)
            
            if sp.name=="a"     // anchor of the term with an id
                j = 2           //  => we skip the anchor
                sp = dt.children(2)
            end
            while sp.name~="span" & j<dt.children.size
                j = j+1
                sp = dt.children(j)
            end
            it = it+1
            tmp = uman_cleantxt(sp.content, "blanks")
            // trimming any ending ":" (as in the ode page)
            L = length(tmp)
            if L>0 & part(tmp,L)==":"
                terms(it)  = part(tmp,1:L-1)
            else
                terms(it) = tmp
            end
            // Updating local headerLength estimation:
            k = find(length(terms)<maxL)
            if k~=[]
                hL = max(length(terms(k)))+3
            else
                hL = minL
            end
            // <dd> DEFINITION
            // If it exists (<listitem> defined), we get all its contents:
            [j, sdt] = (j+1, dt.children.size)
            // looking for the forthcoming <dd> section:
            if convstr(dt.children(j).name)=="dd"
                ddtxt = uman_xml2txt(dt.children(j), lineHead+part(" ",ones(1:hL)))
            else // Contents out of term's <listitem>
                sdt = dt.children.size
                ddtxt = []
                for k = j:sdt
                    tmp = uman_xml2txt(dt.children(k), lineHead+part(" ",ones(1:hL)))
                    ddtxt = [ddtxt ; tmp]
                end
            end
            defs(it) = ddtxt
            //dd = dt.children(j) should be <dd><p class="para">...</p></dd>
            // nothing is expected between </dd> and the embedding </dt>
            i = i+1
        end

        // Building the whole set of <dt> entries
        // --------------------------------------
        // For short terms, the description is appended to term's line :
        // Setting the standard width of items label field
        I = find(length(terms)<maxL)
        if I~=[] then
            nC = max(length(terms(I))) + 3  // +3 for " : "
            terms(I) = justify(terms(I),"l")
        else
            nC = minL
        end
        // Setting the local blank header
        header = part(" ", ones(1:nC))

        // Setting the standard width of item Description field
        wD = max([30, cw - length(lineHead+header) ])

        // Loop on items
        for L = 1:size(terms,1)
            wD_line1 = wD - max([ 0 (length(terms(L))+3-nC)])
            tmp = defs(L)
            if size(tmp,1)<2 then
                tmp2 = uman_wraplines(tmp, wD_line1+1)
                if tmp==[]
                    thistxt =[thistxt; terms(L)]
                else
                    thistxt =[thistxt; terms(L)+" : "+tmp2(1)]
                    if tmp~=tmp2(1) then        // The first line has been wrapped
                        tmp2 = uman_wraplines(strcat(tmp2(2:$)," "), wD)
                        // thistxt  = [thistxt; ""; header + tmp2]
                        if header~=[] & tmp2~=[]
                            tmp2 = header + tmp2
                        end
                        thistxt  = [thistxt; tmp2; ""]
                    end
                end
            else
                if wD_line1 < wD
                    // The first line is kept clear
                    thistxt =[thistxt; terms(L)+" : " ;  header+tmp ]
                else
                    thistxt =[ thistxt ; terms(L)+" : "+tmp(1); header + tmp(2:$)]
                end
            end
            if size(tmp,1)>1 & stripblanks(thistxt($))~="" then
                thistxt($+1) = ""
            end
        end
        txt = " "+thistxt
    // ---------------------------------------------------------------
    elseif or(name==["ul" "ol"])
        isInList = %t
        // Processing all <li> entries :
        litab = []
        if name=="ul" then
            indent = "  "
        else
            indent = "    "
        end
        for i = 1:nc
            li = node.children(i)

            // Summary or See also list: preprocessing
            //  If the item's title has white spaces, it isn't the item's id
            //   => if it has .html & isn't a help section:
            //      replace the title with the id = html filename
            if grep(options, "s")~=[] | secType=="see_also"
                tmp = li.content
                //tmp2 = stripblanks(strsplit(tmp,"—",1)) // item's title
                // We get the related id: item_id.html => item_id
                if length(li.children)>0
                    href = li.children(1).attributes.href
                else
                    href = ""
                end
                // We get the id of the item:
                if grep(href,".html")~=[]  //  href="item.html"
                    id = strsplit(href,".",1)
                    id = id(1)
                    if id~=[] & part(id,1:8)~="section_"
                        // In case of homonymy: we remove the appended hash code:
                        if grep(id, "/.+\-[0-9a-f]{31,32}/","r")
                            id = strsubst(id, "/\-[0-9a-f]{31,32}/","","r")
                        end
                    else
                        id = ""
                    end
                    if id~=""
                        if length(li.children)>=2
                            li.children(1).content = id
                        else
                            li.children(1).content = id + " — " + tmp
                        end
                    end
                elseif grep(href,"scilab://")~=[]
                    // Scilab entries quoted from external modules,
                    //  with href="scilab://item"
                    id = part(href,10:$)
                    if id~=""
                        li.children(1).content = id+" — "+li.children(1).content
                    end
                end
            end
            // --------------
            tmp = uman_xml2txt(li, "  "+lineHead)
            if tmp~=[]
                if name=="ul"
                    tmp(1) = "* "+tmp(1)
                else
                    tmp(1) = sprintf("%2d",i)+". "+tmp(1)
                end
            end
            if size(tmp,1)>2 & stripblanks(tmp($))~="" then
                tmp($+1) = ""
            end
            if size(tmp,"*")>1
                tmp(2:$) = indent + tmp(2:$)
            end
            litab = [ litab ; tmp ]
        end
        if size(litab,1)>8 then
            litab = [""; litab]
        end
        txt = litab
        if txt~=[] then
            txt = " " + txt
        end

        // Post-processing for the list of summaries, in order to align the "--"
        // ---------------------------------------------------------------------
        if (grep(options, "s")~=[] | secType=="see_also") & txt~=[]
            if grep(options, "s")~=[] then
                txt = strsubst(txt, "* percent", "* %")
            end
            K = grep(txt, "/^ \*.*?—/","r")
            Kw = grep(txt, "/^   [^\s\*]/", "r")  // wrapped line
            nL = size(txt,1)
            if K~=[] then
                txt1 = stripblanks(txt(K))
                s = strcspn(txt1,"—")            // locating the "—" separator
                ms = mean(s)
                ss = stdev(s)
                p = min([18, round(ms+ss)])
                if ss<7 then
                    p = min([18, round(ms+1.5*ss)])
                end
                j = find(s>0 & s<p)       // # of lignable lines
                t = []
                for m = j
                    t = [ t ; strsplit(txt1(m),s(m))']
                end
                p = min([18, max(length(t(:,1)))+1])
                if t~=[]
                    t = justify(t(:,1), "l") + " " + t(:,2)
                end
                txt1(j) = t
                txt(K) = txt1
                // Shifting wrapped lines
                head = part(" ", ones(1,p+3))
                ww = max([30, cw - length(lineHead)-p]) // wrapping width

                newtxt = []
                line = ""
                for i = 1:size(txt,"*")
                    if or(i==Kw)
                        line = line + " "+ stripblanks(txt(i))
                    else
                        if line~=""
                            // we process the registered line
                            tmp = uman_wraplines(line, ww+p+3)
                            if tmp~=[]
                                newtxt = [ newtxt ; " "+tmp(1) ]
                            end
                            if size(tmp,1)>1
                                tmp = uman_wraplines(strcat(tmp(2:$), " "), ww)
                                newtxt = [newtxt ; head + tmp ]
                            end
                            line = " "
                        end
                        if or(i==K)
                            line = " "+txt(i)
                        else
                            newtxt = [newtxt ; txt(i)]
                        end
                    end
                end
                if line~=" "
                    // we process the registered line
                    tmp = uman_wraplines(line, ww+p+3)
                    newtxt = [ newtxt ; " "+tmp(1) ]
                    if size(tmp,1)>1
                        tmp = uman_wraplines(strcat(tmp(2:$), " "), ww)
                        newtxt = [newtxt ; head + tmp ]
                    end
                end
            else
                newtxt = txt
            end
            txt = newtxt
        end
//     ---------------------------------------------------------------
    elseif or(name==["table" "tbody"])
        // <note> and <warning> are in a table starting with an image:
        //   <table><tr><td valign="top"><img src="ScilabNote.png"/>
        if length(node.children)==0
            txt = ""
            return
        end
        tmp = node.children(1).children(1).children
        if tmp.size>0 & tmp(1).name=="img"
            tmp = tmp(1).attributes.src
            if grep(tmp, ["ScilabNote.png" "ScilabTip.png"])~=[]
                headLine = convstr(_("Note"),"u")+" : "
            elseif grep(tmp, ["ScilabWarning.png" "ScilabCaution.png"])~=[]
                headLine = convstr(_("Warning"),"u")+" : "
            elseif grep(tmp, "ScilabImportant.png")~=[]
                headLine = _("uman", "IMPORTANT: ")
            else
                headLine = ""
            end
//            txt = txt + uman_cleantxt(node.content+" ","blanks")
//            txt = uman_wraplines(txt, max(30, cw-length(lineHead)))
//            return
            node.children(1).children(1) = [];
        else
            headLine = ""
        end
        a = node.attributes
        TABLE = %t
        Array = []
        for i = 1:nc        // Loop over rows
            row = uman_xml2txt(node.children(i))
            if i>1
                // missing cells?
                if size(row,2)< size(Array,2)
                    row = resize_matrix(row, 1, size(Array,2))
                elseif size(row,2)> size(Array,2)
                    Array = resize_matrix(Array, 1, size(row,2))
                end
            end
            Array = [Array ; row ]
        end
        // If there are empty (or blank) columns => we remove them
        L = length(stripblanks(strcat(Array,"","r")))
        Array(:, find(L==0)) = []

        // Useful parameters needed in wraptable
        ncols = size(Array,2)   // Number of columns
        withUmanBorders = (a.border~=[] & a.border~="0")

        // We wrap long lines
        global umanAllowWideTables
        if umanAllowWideTables=="checked" & headLine==""
            cw = lines(); cw = cw(1)
        end
        [Array, cw] = uman_wraptable(Array, max(30, cw-length(lineHead))-1)
        // cw: columns widths

        // Processing Colspan, justification, and columns borders:
        // ------------------------------------------------------
        colspanLines = grep("|"+strcat(Array,"|","c"), "/\|\d+§/", "r")
        ok = setdiff(1:size(Array,1), colspanLines) // rows without colspan
        // Regular rows without colspan:
        Array(ok,:) = justify(Array(ok,:), "l")
        if withUmanBorders
            csep = "| "     // columns separator
            csepL = 2       // csep length
        else
            csep = " "
            csepL = 1
        end
        if Array(ok,2:$)~=[]
            Array(ok,2:$) = csep + Array(ok,2:$)
        end

        // Colspan HERE         // Protéger "§" => "&...;" en amont
        // ............
        if colspanLines~=[]
            colSpannedROWS = list()  // h-concatenated colspanned (bordered) rows
            cw = max(length(Array(ok,:)), "r")
            for i = colspanLines
                row = Array(i,:)
                Tcells = list()  
                wi = []          // actual cells widths after spanning
                nbr = []         // nb. of rows in spanned cells after wrapping
                j = 1
                while j <= ncols
                    c = row(j)  // current <td> cell
                    if grep(c, "/\d+§/", "r")==[]
                        // No colspan:
                        if j==1
                            wi = cw(1)
                        else
                            c = csep + c
                            wi = [wi cw(j)]
                        end
                        Tcells($+1) = c
                    else
                        // It is colspanned:
                        c = tokens(c, "§")
                        nbs = evstr(c(1)) // number of spanned columns
                        tspanw = sum(cw(j:(j+nbs-1))) // total spanwidth
                        wi = [wi tspanw]
                        c = strcat(c(2:$), "§")
                        if j>1
                            c = uman_wraplines(c, tspanw-csepL)
                            c = csep + c
                        else
                            c = uman_wraplines(c, tspanw)
                        end
                        Tcells($+1) = c
                        j = j + nbs-1
                    end
                    nbr = [nbr size(c,1)]
                    j = j + 1
                end
                // equalizing the number of rows in cells, and justifying them
                Nbr = max(nbr)
                newRows = ""
                for j = 1:size(Tcells)
                    c = Tcells(j)
                    tmp = []
                    // Padding vertically
                    if nbr(j)<Nbr
                        tmp = emptystr(Nbr-nbr(j),1)
                        if j>1
                            tmp = csep + tmp
                        end
                    end
                    c = [c ; tmp]
                    c = part(c, 1:wi(j))    // Padding horizontally:
                    newRows = newRows + c
                end
                colSpannedROWS(i) = newRows
                // Substitution in Array is done herebelow after h-concatenation of cells
            end
        end

        // ............
        Array = strcat(Array, "", "c")
        // Substitution of groups of rows with spanned columns to related Array rows:
        if colspanLines~=[]
            newArray = []
            for i = 1:size(Array,1)
                if ~or(colspanLines==i)
                    newArray = [newArray ; Array(i)]
                else
                    newArray = [newArray ; colSpannedROWS(i)]
                end
            end
            Array = newArray
        end
        // External borders:
        if ~withUmanBorders
            if size(Array,1)>1 then
                txt = " " + Array
            else
                txt = stripblanks(Array)
            end
        else
            Array = "| " + Array + " |"
            hline = part("-",ones(1,length(Array(1))*RelCharWidth))
            txt = " " + [hline ; Array ; hline ]
        end
        // For NOTE, WARNING, and IMPORTANT tags:
        if isdef("headLine","l") & headLine~=[]
            if size(txt,1)==1
                txt = headLine + txt;
            else
                txt = [headLine ; txt];
            end
        end
        if secType=="history" & isdef("isInList") & isInList
            txt = "¤" + txt
        end

    // ---------------------------------------------------------------
    elseif name=="tr"
        Row = node.children
        row = []
        if nc<=1  // Only one column of cells
            txt = uman_xml2txt(Row(1), lineHead)
        else
            isInCell = %t  // used to avoid wrapping lines too early
                           //             (done later in wrap_table)
            // Loop over cells of the row:
            for i = 1:nc
                row = [row ; uman_xml2txt(Row(i),"")]
            end
            txt = row'
        end
    // ---------------------------------------------------------------
    elseif or(name==["td" "th"])
        if nc==0 & node.content==""
            txt = ""
        else
            if nc==1 & node.children.name=="text"
                txt = uman_cleantxt(node.content+" ","blanks")
            else                                   // itemized lists in history
                isInCell = %t
                tmp = []
                thisline = ""
                for i = 1:nc
                    [tmp2, inline] = uman_xml2txt(node.children(i),"")
                    if inline & tmp==[]
                        if tmp2<>[]
                            thisline = thisline + tmp2
                        end
                    else
                        tmp = [tmp ; thisline ; tmp2]
                        tmp(tmp=="") = []
                        thisline = ""
                    end
                end
                if tmp==[]
                    tmp = thisline
                end
                // We remove possible heading empty rows
                for i=1:size(tmp,1)
                    if stripblanks(tmp(i))==""
                        tmp(i) = "!delete!"
                    else
                        break
                    end
                end
                tmp(tmp=="!delete!") = []

                // We concatenate all rows with "€" to keep only one row per cell
                // We will expand it only in wrap_table.
                txt = strcat(tmp, "€")
            end
        end
        // Tagging any colspan in head of the cell
        k = find(convstr(xmlName(node.attributes))=="colspan")
        if k~=[]
            //pause
            tmp = node.attributes(k)
            if txt==[]
                txt = " "
            end
            txt = tmp + "§" + txt
            txt = [txt ; emptystr(evstr(tmp)-1,1)]
        end

    // ---------------------------------------------------------------
    else
        txt = "<"+name+">"+uman_cleantxt(node.content+" ","blanks")+"</"+name+">"
    end
endfunction

// ------------------------------------------------------------------

// Copyright (C)  2013 - 2020   Samuel GOUGEON
// Utility function for uman()
function uman_openURL(url, msg)
    if sciVer > 6.01
        v = xmlGetValues("//web/body/web", ["default-browser" "command-browser"])
    else
        v = getPreferencesValue("//web/body/web", ["default-browser" "command-browser"]);
    end
    [default, cmd] = (v(1)=="true", v(2))
    select getos()
    case "Windows"
        if default
            winopen(url)
        else
            unix(msprintf("start """" %s ""%s""", v(2), url))
        end
    case "Linux"
        if default, cmd = "xdg-open ", end
        unix(cmd + """" + url + """")
    case "Darwin"
        if default, cmd = "open ", end
        unix(cmd + """" + url + """")
    else
        warning(msg)
    end
endfunction


// ------------------------------------------------------------------

// Copyright (C)  2013 - 2022   Samuel GOUGEON
// Utility function for uman()
function [item , options] = uman_get_equiv(item, options, equivType)
// item: the item to search
// List of equivalences or nearest redirections
//  column #1: external item (Octave, etc...).
//  column #2: Scilab redirection (help's filename).
//             "£" prefix = options to be passed when calling uman (default = s)
// equivType : "shortcuts" | "ex2in" | "ex2ex" | "exin2in"

    // Redirections inner => inner:
    // The left entry must not be a legal Scilab entry
    // The right entry must be a legal Scilab entry
    shortcuts = [
      "Atoms"    "pd£atoms"
      "colors"   "£color"           // inner redirection | "£color_list" si "w"
      "event"   "pdeh£xclick"
      "events"  "pdeh£xclick"
      "files"   "£listfiles"
      "plotting" "£Graphics"
      "gui"     "£clipboard"
      "hdf5"    "£h5read"           // http://www.hdfgroup.org/HDF5/
      "history" "£gethistoryfile"   // inner summary
      "keys"    "pd£console"        // inner redirection
      "latex"   "pd£math_rendering_features_in_graphic" // inner redirection
      "lenght"  "length"            // common mis-spelling
      "overload" "overloading"
      "signal"   "£detrend"
      "stats"    "£reglin"
      "trigo"    "£cos"
      "types"    "pd£type"          // inner redirection with "pd" forced
      "variables" "£who"
      "windows"  "£winopen"
      "xml"     "£xmlRead"
      ]

    // External => Scilab redirections
    // In ScilabFirst mode as long as the left entry is not a legal Scilab entry
    ex2in = [
        "all"     "and"
        "angle"   "atan"
        "any"     "or"
        "argv"    "sciargs"       // Octave
        "arrayfun" "feval"
        "atan2"   "atan"
        "axes"    "newaxes"
        "axis"    "replot"
        "balance" "balanc"
        "bartlett" "window"
        "bicg"    "conjgrad"
        "bicgstab" "conjgrad"
        "blackman" "window"
        "blkdiag" "blockdiag"
        "box"     "gca"
        "caxis"   "colorbar"
        "celldisp" "disp"
        "cgs"     "conjgrad"
        "charpoly" "poly"
        "chebwin" "window"
        "cla"     "delete"
        "class"   "typeof"
        "clear global"    "clearglobal"
        "closereq" "xdel"
        "colormap" "jetcolormap"
        "columns"  "size"                     // Octave
        "commandhistory"  "browsehistory"
        "compan"   "companion"
        "completion_matches"  "completion"    // Octave
        "computer" "getos"
        "contains" "grep"
        "contourc" "contour2di"
        "contour3" "contour"
        "cot"     "cotg"
        "copyobj" "copy"
        "cputime" "timer"
        "csvread" "csvRead"
        "dbclear" "debug"
        "dbcont"  "resume"
        "dbquit"  "abort"
        "dbstack" "where"
        "dbstatus" "debug"
        "dbstop"  "debug"
        "dbup"    "return"
        "deal"    "brackets"
        "deblank" "stripblanks"
        "demo"    "demo_gui"
        "dlmwrite" "fprintfMat"
        "echo"    "mode"
        "eig"     "spec"
        "ellipj"  "amell"
        "ellipke" "percentk"
        "endsWith" "grep"
        "eps"     "percenteps"
        "erase"   "strsubst"
        "errorbar" "errbar"
        "errordlg" "messagebox"
        "exist"   "exists"
        "extractAfter" "regexp"
        "extractBefore" "regexp"
        "false"   "percentf"
        "fclose"  "mclose"
        "feof"    "meof"
        "fftn"    "fft"
        "fgetl"   "mgetl"
        "fgets"   "mgetl"
        "fill"    "xfpolys"
        "flintmax" "number_properties"
        "flip"    "flipdim"
        "flipud"  "flipdim"
        "fliplr"  "flipdim"
        "fopen"   "mopen"
        "fplot"   "plot"
        "fputs"   "mputstr"       // Octave
        "fread"   "mget"
        "frewind" "mseek"
        "fseek"   "mseek"
        "func2str" "macr2tree"
        "fwrite"  "mput"
        "fzero"   "fsolve"
        "gallery" "testmatrix"
        "gammainc" "cdfgam"
        "get_home_directory"  "home"  // Octave
        "getpref" "xmlGetValues"
        "ginput"  "locate"
        "grid"    "xgrid"
        "gtext"   "legends"
        "hamming" "window"
        "hann"    "window"
        "hanning" "window"        // Octave
        "help"    "uman"          // Octave
        "helpdlg" "messagebox"
        "helpwin" "help"
        "hggroup" "glue"
        "hgload"  "load"
        "hgsave"  "save"
        "hist"    "histplot"
        "histcounts" "histc"
        "hold"    "axes_properties"
        "housh"   "householder"   // Octave
        "horzcat" "cat"
        "i"       "percenti"
        "ifft2"   "ifft"
        "ifftn"   "ifft"
        "image"   "Matplot"
        "imagesc" "Matplot1"
        "index"   "strindex"      // Octave
        "inf"     "percentinf"
        "int2str" "string"
        "intmin"  "int8"
        "intmax"  "int8"
        "invfeqz" "frfit"
        "is_absolute_filename" "is_absolute_path" // Octave
        "isa"     "m2sci_isa"
        "isalnum" "isalphanum"    // Octave
        "iscell"  "typeof"
        "iscellstr" "typeof"
        "isfolder"  "isdir"
        "isgraphics" "is_handle_valid"    // Octave
        "isguirunning" "getscilabmode"    // Octave
        "ishghandle" "is_handle_valid"
        "isinteger" "type"
        "isjava"  "typeof"
        "iskeyword" "getscilabkeywords"
        "ismac"   "getos"
        "ismember" "members"
        "ispc"    "getos"
        "isunix"  "getos"
        "is_valid_file_id" "file" // Octave
        "j"       "percenti"
        "join"    "strcat"
        "kaiser"  "window"
        "keyboard" "pause"
        "listdlg" "x_choose"
        "lookfor" "apropos"
        "lower"   "convstr"
        "lstat"   "fileinfo"      // Octave
        "magic"   "testmatrix"
        "mat2str" "sci2exp"
        "matlabroot" "SCI"
        "msgbox"  "messagebox"
        "mfilename"  "get_absolute_file_path"
        "mod"     "pmodulo"
        "more"    "lines"
        "nan"     "percentnan"
        "nanmin"  "min"
        "nanmax"  "max"
        "nargin"  "argn"
        "nargout" "argn"
        "newline" "ascii"
        "nonzeros" "spget"
        "num2str" "string"
        "numel"   "size"
        "OCTAVE_HOME" "SCIHOME"   // Octave
        "ode23"   "ode"
        "ode45"   "ode"
        "ode113"  "ode"
        "ode15s"  "ode"
        "odeget"  "odeoptions"
        "openvar" "editvar"
        "paddarray" "resize_matrix"
        "patch"   "xfpolys"
        "pcg"     "conjgrad"
        "pi"      "percentpi"
        "pkg"     "atoms"         // Octave
        "planerot" "givens"
        "plot3"   "param3d"
        "plottools" "ged"
        "polar"   "polarplot"
        "polyarea" "polygonmetrics"
        "polyder" "derivat"
        "polygcd" "gcd"           // Octave
        "polyval" "horner"
        "prctile" "perctl"
        "printdlg" "toprint"
        "psi"     "dlgamma"
        "puts"    "mprintf"
        "questdlg" "messagebox"
        "quiver"  "champ"
        "rande"   "grand"         // Octave
        "randg"   "grand"
        "randi"   "grand"
        "randn"   "rand"
        "randp"   "grand"         // Octave
        "randperm" "grand"
        "range"   "strange"
        "residue" "pfss"
        "rbbox"   "rubberbox"
        "realmax" "number_properties"
        "realmin" "number_properties"
        "regexprep"  "strsubst"
        "rem"     "modulo"
        "replace" "strsubst"
        "resample" "intdec"
        "reshape" "matrix"
        "reverse" "strrev"
        "rmfield" "null"
        "rotate3d" "rotate_axes"
        "rot90"   "flipdim"
        "rows"    "size"          // Octave
        "run"      "exec"
        "shg"      "show_window"
        "skrewness" "cmoment"     // Octave
        "sort"     "gsort"
        "sortrows" "gsort"
        "source"   "exec"         // Octave
        "spalloc"  "spzeros"
        "spdiags"  "diag"
        "splitlines" "strsplit"
        "sprandn"  "sprand"
        "sprandsym" "sprand"
        "spy"      "PlotSparse"
        "stairs"   "plot2d2"
        "startsWith" "grep"
        "stat"     "fileinfo"     // Octave
        "std"      "stdev"
        "stdin"    "percentio"    // Octave
        "stdout"   "percentio"    // Octave
        "stderr"   "percentio"    // Octave
        "stem"     "plot2d3"
        "str2func" "execstr"
        "strcmp"   "comparison"
        "strcmpi"  "strcmp"
        "strfind"  "strindex"
        "strings"  "emptystr"
        "strjoin"  "strcat"
        "strjust"  "justify"
        "strlength" "length"
        "strread"  "msscanf"
        "strrep"   "strsubst"
        "strtrim"  "stripblanks"
        "strtrunc" "strncpy"      // Octave
        "substr"   "part"
        "surf2patch" "genfac3d"
        "sylvester"  "sylv"
        "system"   "unix_g"
        "switch"   "select"
        "tempdir"  "TMPDIR"
        "terminal_size" "lines"   // Octave
        "texlabel" "math_rendering_features_in_graphic"
        "text"     "xstring"
        "textscan" "msprintf"
        "time"     "getdate"      // Octave
        "tolower"  "convstr"      // Octave
        "toupper"  "convstr"      // Octave
        "trapz"    "inttrap"
        "true"    "percentt"
        "tukeywin" "window"
        "typeinfo"   "typeof"     // Octave
        "uisetcolor" "uigetcolor"
        "uitable" "uicontrol"
        "unix"    "unix_g"
        "upper"   "convstr"
        "urlread" "getURL"
        "var"     "variance"
        "version" "getversion"
        "vertcat" "cat"
        "waitforbuttonpress" "xclick"
        "warndlg" "messagebox"
        "webread" "getURL"
        "which"   "get_function_path"
        "whos -file" "listvarinfile"
        "xlim"    "axes_properties"
        "ylim"    "axes_properties"
        "yyaxis"  "multiscaled_plot"
        "zlim"    "axes_properties"
        "zoom"    "zoom_rect"
    ]
//    if v(1)>6  | v(1)==6 & v(2)>0 then
    if sciVer >= 6.1
        ex2in = [ ex2in ;
            "loglog"  "plot"
            "semilogx" "plot"
            "semilogy" "plot"
            ]
    else
        ex2in = [ ex2in ;
            "loglog"  "plot2d"
            "semilogx" "plot2d"
            "semilogy" "plot2d"
            ]
    end

    ex2key = [
      "view"    "rotation_angles"  // for "g", "w"
    ]
    // REDIRECTIONS to modules, FileExchanges, or external references
    // In ScilabFirst mode AFTER search in Scilab JAR & search in modules JAR
    // as long as the left entry is not a legal Scilab entry.
    // "£" means "summary of"
    ex2ex = [
      "expm1"       "specfun"
      "Firebird"    "scidb"
      "help"        "uman"
      "inpolygon"   "point_in_polygon"
      "lambertw"    "specfun"
      "log1p"       "specfun"
      "MySQL"       "scidb"
      "MSSQL"       "scidb"
      "ODBC"        "scidb"
      "Overload"    "Overload_Toolbox"
      "pascal"      "specfun"
      "PostgreSQL"  "scidb"
      "print_usage" "disp_usage"
      "python"      "PIMS"
      "signbit"     "floatingpoint"
      "Sqlite"      "scidb"
      "web"         "browseURL"
      "apifun"      "£apifun_overview"          // if Apifun installed
      "cglab"       "£constrained_delaunay_2"   // if CGLAB installed
      "ipcv"        "£section_06db514eed021798dcf04f7718e0d829" // if IPCV installed
      "metanet"     "£bandwr"                   // "uman metanet s" will be OK
      "scimax"      "£PERCENTedispflag"         // if SciMax installed
      ]

    // Queries matching a scilab item but with a different meaning in an external
    //  language. There are considered only if the priority target is external package
    exin2in = [
      "contour" "contour2d"
      "conv"    "convol"
      "dos"     "unix_g"
      "end"     "dollar"
      "home"    "d£tohome"      // poor page => force "d"
      "load"    "loadmatfile"
      "length"  "size"
      "moment"  "cmoment"
      "null"    "kernel"
      "pause"   "halt"
      "quiver"  "champ"         // quiver implemented in plotlib
      "square"  "squarewave"
      "system"  "unix_g"
      "type"    "unix_w"
      "ver"     "getversion"
      ]

   item = item(1)
    if or(equivType==["shortcuts" "ex2in" "ex2ex" "exin2in" "ex2key"])
        execstr("e="+equivType)
        if convstr(item)==item
            ie = find(convstr(e(:,1))==item)
        else
            ie = find(e(:,1)==item)
        end
        if ie~=[]
            item = e(ie(1),2)
            [item, options] = uman_updateQueryFromEquiv(item, options)
        end
    end
endfunction

// -------------------------------------------------------------------------
// Copyright (C)  2017-   Samuel GOUGEON
// Utility function for uman()

function atomsModules = uman_get_actual_atoms_list()
    // returns the matrix of ["a" modulename forgename website]
    // of ATOMS packages available for this computer (and this version
    // of Scilab), extracted from the ATOMS packages file

    atomsModules = []

    // We look for an ATOMS packages file :
    packagesFile = "SCIHOME/.atoms/*/packages"
    if sciVer > 7 then
        packagesFile = packagesFile + ".json"
    end
    d = dir(packagesFile)

    if isempty(d) then
        if sciVer > 7
            packagesFile = "SCI/.atoms/packages.json"
        else
             packagesFile = "SCI/.atoms/packages"
        end
        d = dir(packagesFile)
    end
    if ~isempty(d) then
        packagesFile = d.name
        tmp = fileinfo(packagesFile)
        Da = msprintf("%4d-%02d-%02d %02d:%02d:%02d", tmp([1 2 6:9]))
    else
        // no ATOMS packages file.
        // We could try to download it
//        global umanAllowDownloadAtomsPackages
//        if isdef("umanAllowDownloadAtomsPackages") & ..
//           umanAllowDownloadAtomsPackages
//            try
//                [?,atomsP] = getURL("https://atoms.scilab.org/")
//                // yields an errors when there is no active internet connexion
//                if atomsP~=[]
//                    [?,?,?,f] = regexp(atomsP,"|\/toolboxes\/([^\/]+)\/|")
//                    if f~=[] then
//                        [?,k] = gsort(convstr(f(:,1)),"g","i")
//                        f = f(k)
//                        f(1,2) = ""  // to create empty column #2
//                        atomsModules = f
//                    end
//                    return
//                end
//            end
//            // no internet connexion
//        end
        return
    end
    // Now we get the archived uman packages list.
    // If it is older than the ATOMS list, we reextract it from ATOMS.
    // Otherwise we use the archive (~ 50x faster)
    umanAtomsPackages = SCIHOME+"/uman/atomsPackages.csv"
    relistPackages = ~isfile(umanAtomsPackages)
    if ~relistPackages then
        tmp = fileinfo(umanAtomsPackages)
        Du = msprintf("%4d-%02d-%02d %02d:%02d:%02d", tmp([1 2 6:9]))
        relistPackages = strcmp(Da, Du)>0
    end

    if ~relistPackages then
        atomsModules = csvRead(umanAtomsPackages, ";", [], "string")
    else
        if sciVer > 7 then
            p = fromJSON(packagesFile, "file");
            packages = p.packages;
        else
            load(packagesFile)
        end
        forgePattern = "|^https{0,1}\:\/\/forge\.scilab\.org\/.*\/(.+)\/|"
        modules= fieldnames(packages)
        atomsModules = []
        for i = 1:size(modules,1)
            name = modules(i,1)    // module id (name)
            m = packages(name)
            v = fieldnames(m)   // versions
            m = m(v(1))         // last version
            if length(m) <> 0  // For some reason, some packages (Cardiovascular) are 0x0 struct
                f = m.WebSite
                if f~=""
                    [?,?,?,f] = regexp(m.WebSite, forgePattern);
                end
                atomsModules(i,:) = ["a", name, f, m.WebSite]
            end
        end
        csvWrite(atomsModules, umanAtomsPackages, ";")
    end
endfunction

// -------------------------------------------------------------------------

// Copyright (C)  2013 - 2022   Samuel GOUGEON
// Utility function for uman()
function table = uman_get_atoms_forge()
    // Actually, the table may also contain references of non-ATOMS packages
    // [ type atoms_id  forge_id  website ]
    atoms = [
    "a" "accsum"            "accsum"    ""
    "a" "AdaptiveOptics"    ""          "http://sciao.sourceforge.net/#authors"
    "a" "ADS_CoLiSyS"       ""          ""
    "a" "aerospace-toolbox" "aerospace-toolbox"	""
    "a" "aerospace_blockset" "aerospace-blockset"	""
    "a" "ampl_toolbox"      ""          "http://code.google.com/p/scilab-mip"
    "a" "Analog_and_DigitalCommunication"   ""  "http://www.geocities.ws/senthilirtt/"
    "a" "Anfislab"          ""          "http://www.tritytech.com"
    "a" "ANN_Toolbox"       "ann-toolbox" ""
    "a" "apifun"            "apifun"    ""
    "a" "ardronemodule"     ""          "https://code.google.com/p/ardronetoolbox/"
    "a" "arduino"           "arduino"   ""
    "a" "arfit"             ""          "http://www.gps.caltech.edu/~tapio/arfit/"
    "a" "assert"            "assert"    ""
    "a" "BackDoor"          "BackDoor"  ""
    "a" "BTD"               ""          "http://dcb.res.in/scilab-btd/"
    "a" "casci"             "casci"     "http://philippe.castagliola.free.fr"
    "a" "CCA"               ""          "http://people.ok.ubc.ca/ylucet/cca.html"
    "a" "celestlab"         "celestlab" ""
    "a" "celestlabx"        ""          ""
    "a" "CEMEF-UTN"        ""           ""
    "a" "cglab"             "cgal"      ""
    "a" "CLUSTER"           ""          "http://www.xing.com/profile/Harald_Galda"
    "a" "Computer-Vision"   "cvfunctions"  ""
    "a" "condnb"            "condnb"    ""
    "a" "Conint"            "conint"    ""
    "a" "conmin"            ""          "http://code.google.com/p/scilab-mip"
    "a" "convertlatin"      "convertlatin"  ""
    "a" "coselica"          "coselica"  ""
    "a" "CMA-ES"            ""          "http://www.cfuttrup.com"
    "a" "CPGE"              "cpge"      "http://www.demosciences.fr/"
    "a" "csv_readwrite"     "csv-readwrite" ""
    "a" "Curl"              ""          ""
    "a" "CUTEr"             "cuter"     ""
    "a" "CWA"               "CWA"       ""
    "a" "dace_scilab"       "dace-scilab"   ""
    "a" "dataint"           ""          ""
    "a" "dbldbl"            "dbldbl"    ""
    "a" "dde_toolbox"       "dde-toolbox"   ""
    "a" "DD_QD"             ""          "http://www.mi.kagu.tus.ac.jp/qupat.html"
    "a" "DeCaToKi"          ""          "http://sites.google.com/site/ronaystein/"
    "a" "diffcode"          "diffcode"  ""
    "a" "digimetrie_Tools"  ""          "http://www.digimetrie.com/"
    "a" "dispmat"           "dispmat"   ""
    "a" "distfun"           "distfun"   ""
    "a" "Dynpeak"           ""          ""
    "a" "edflib"            ""          "http://www.teuniz.net/edflib/"
    "a" "emd_toolbox"       ""          "http://perso.ens-lyon.fr/patrick.flandrin/emd.html"
    "a" "EMTTOOL"           ""          ""
    "a" "E4Coder"           ""          "http://www.e4coder.com/"
    "a" "FACT"              ""          ""
    "a" "favouritedirs"     ""          ""
    "a" "femtruss"          ""          "http://code.google.com/p/scilab-mip/"
    "a" "Financial"         "Financial" ""
    "a" "FireBirdToolboxforFireBirdRobot"   ""  "http://www.nex-robotics.com/"
    "a" "fitters"           ""          ""
    "a" "floatingpoint"     "floatingpoint" ""
    "a" "fmincon"           "fmincont"  ""
    "a" "FOT"               ""          "http://www.scilab.in/fossee-scilab-toolbox/optimization-toolbox"
    "a" "fsmtbx"            ""          "http://fsmtbx.origo.ethz.ch/"
    "a" "getDmatrix"        ""          "http://uolp.unifor.br/oul/pages/academico/posgraduacao/novoSite/professorPL.jsp?Professor=3820065&p_tipo_pagina=grad"
    "a" "Global_Optim_toolbox"  ""      "http://www.sc.iitb.ac.in/~bhagyesh/Scilab/GlobalOptimizationToolbox.html"
    "a" "grocer"            ""          "http://dubois.ensae.net/grocer.html"
    "a" "guibuilder"        ""          "http://www.tritytech.com/"
    "a" "guimaker"          "guimaker"  ""
    "a" "gui2bitmap"        ""          ""
    "a" "helptbx"           "helptbx"   ""
    "a" "huffcomp"          ""          ""
    "a" "hypt"              "hypt"      ""
    "a" "hyb_auto_module"   "hyb-auto-module"   ""
    "a" "HYDROGRv50"        ""          "http://lerat.julien.free.fr/CONT_FR/HYDROGR_FR.html"
    "a" "ica_toolbox"       ""          "http://research.ics.tkk.fi/ica/fastica/"
    "a" "identification"    ""          ""
    "a" "imsls"             "imsls"     ""
    "a" "indinf"            "indinf"    ""
    "a" "intprbs"           "intprbs"   ""
    "a" "iodelay"           ""          ""
    "a" "IPD"               "IPD"       "http://www.xing.com/profile/Harald_Galda/"
    "a" "ipoptlib"          "ipoptlib"  ""
    "a" "IPT2"              ""          "http://cdac.in"
    "a" "IPT3"              ""          ""
    //"a" "IWT"               ""          "http://www.cdac.in": replaced with the next
    "a" "Integer_Wavelet_Transform_ToolBox" ""  ""
    "a" "JIMS"              "JIMS"      ""
    "a" "json"              "json"      ""
    "a" "krisp"             ""          ""
    "a" "lcc_windows"       "lcc-windows"   ""
    "a" "leftwavedenoise"   ""          "http://cas.ensmp.fr/~chaplais/fr/Ondelettes/Ondelettes/DebruitageOndelettes.html"
    "a" "libsvm"            "libsvm"    ""
    "a" "linalg"            "linalg"    ""
    "a" "lolimot"           "lolimot"   "http://code.google.com/p/scilab-mip"
    "a" "lowdisc"           "lowdisc"   ""
    "a" "lsitbx"            ""          "http://lsitbx.origo.ethz.ch/"
    "a" "makematrix"        "makematrix"    ""
    "a" "maple2scilab"      "maple2scilab"  ""
    "a" "Mascaret"          ""          "http://www.openmascaret.org/"
    "a" "Mathieu"           "mathieu"   ""
    "a" "MatrixMarket"      "matmark"   ""
    "a" "Maxplus"           ""          ""
    "a" "maxplus_petrinet"  ""          ""
    "a" "Max_Plus_Algebra"  ""          "http://www.its.ac.id/personal/index.php?id=Subiono"
    "a" "MDPtoolbox"        ""          "http://www.inra.fr/mia/T/MDPtoolbox"
    "a" "MED"               "MED"       ""
    "a" "metanet"           "metanet"   ""
    "a" "mi"                ""  "http://penglab.janelia.org/proj/mRMR/"
    "a" "microdaq"          ""          "http://www.microdaq.org"
    "a" "microwave"         ""          "http://www.reveyrand.fr/uW.html"
    "a" "mingw"             "MinGW-toolbox" ""
    "a" "minphase_toolbox"  ""          ""
    "a" "mkltools"          "mkltools"  ""
    "a" "moc_toolbox"       "moc-toolbox" ""
    "a" "modbus"            "modbus"    ""
    "a" "module_lycee"      "module-lycee"  ""
    "a" "montesci"          ""          "http://www.drivehq.com/web/ka_eins/montesci/"
    "a" "MPScilab"          "mpscilab"  ""
    "a" "nan"               "nan-toolbox"   ""
    "a" "NARVAL"            ""          "http://wwwfr.uni.lu/snt/"
    "a" "neuralnetwork"     ""          "http://www.tritytech.com/"
    "a" "NIDAQ"             "nidaq"     ""
    "a" "NISP"              "nisp"      ""
    "a" "nistdataset"       "nistdataset"   ""
    "a" "Nqueens"           ""          ""
    "a" "NTG"               ""          "http://wwwfr.uni.lu/interdisciplinary_centre_for_security_reliability_and_trust/people/foued_melakessou"
    "a" "number"            "number"    ""
    "a" "odesolver"         ""          ""
    "a" "OFIP"              ""          ""
    "a" "opc_client"        "opc-client" ""
    "a" "OpenRTDynamics"    ""          ""
    "a" "opti"              ""          ""
    "a" "optkelley"         "optkelley" ""
    "a" "Ortpol"            "ortpol"    ""
    "a" "Overload_Toolbox"  "overload"  ""
    "a" "PIMS"              "pims"      ""
    "a" "plotlib"           "plotlib"   ""
    "a" "plotplots"         ""          ""
    "a" "portaudio"         "portaudio" ""
    "a" "predictivemodule"  ""          ""
    "a" "ProActiveConnector" ""         "http://proactive.activeeon.com"
    "a" "PSO"               "pso-toolbox" ""
    "a" "PVM"               "pvm"       ""
    "a" "quapro"            "quapro"    ""
    "a" "quickplot"         ""          ""
    "a" "rdataset"          "rdataset"  ""
    "a" "regtools"          "regtools"  ""
    "a" "removed"           ""          ""
    "a" "rest_client"       ""          ""
    "a" "RLDesign"          ""          ""
    "a" "rltool"            ""          ""
    "a" "Robotics_Toolbox"  ""          ""
    "a" "scetoexe"          "scetoexe"  ""
    "a" "Scholar_Lycee_in_English"  "module-lycee"  ""
    "a" "sci_sundials"       ""         "https://gitlab.com/mottelet/sci-sundials"
    "a" "scibench"          "scibench"  ""
    "a" "scicheckupdate"    "scicheckupdate"    ""
    "a" "scicobyla"         "scicobyla" ""
    "a" "SciCrypt"          "scicrypt"  ""
    "a" "SciCSDP"           ""          "http://code.google.com/p/scilab-mip/"
    "a" "scicv"             "scicv"     ""
    "a" "scidb"             "scidb"     ""
    "a" "scidemo"           "scidemo"   ""
    "a" "scidoe"            "scidoe"    ""
    "a" "sciFLT"            ""          "http://www.geocities.ws/jaime_urzua/sciFLT/sciflt.html"
    "a" "SciFreeFEM"        ""          "http://code.google.com/p/scilab-mip/"
    "a" "SciGLPK"           ""          "http://code.google.com/p/scilab-mip/"
    "a" "sciGPGPU"          ""          ""
    "a" "scilab2c"          "scilab2c"  ""
    "a" "Scilab_SSE"        ""          "https://www.scilab.org"
    "a" "Scilab_XLL"        "scilab-xll"    ""
    "a" "scilib_geo"        ""          "http://scilib-geo.area7.de/"
    "a" "SciLPSolve"        ""          "http://code.google.com/p/scilab-mip/"
    "a" "scinetcdf"         "netcdf4"   ""
    "a" "scinlopt"          ""          "http://code.google.com/p/scilab-mip"
    "a" "sciopustc"         ""          "http://opus-project.fr/"
    "a" "scipad"            ""          "http://sourceforge.net/projects/scipad/"
    "a" "scipython"         "pims"      ""
    "a" "sciTorch"          ""          "https://github.com/tanchinluh/sciTorch"
    "a" "sci_ipopt"         "sci-ipopt" ""
    "a" "Scholar_Lycee_in_English"  ""  "http://scilab-scholar.blogspot.com"
    "a" "SegmentedTurningDesign"    ""  ""
    "a" "serial"            ""          "github.com/sengupta/Scilab-Serial"
    "a" "serialport"        ""          ""
    "a" "SIMM"              "SIMM"      ""
    "a" "simplex"           ""          "http://code.google.com/p/scilab-mip"
    "a" "SIP"               "SIP"       "http://siptoolbox.sourceforge.net"
    "a" "siseli"            ""          "http://www.askrprojects.net/software/siseli/index.html"
    "a" "SIVP"              ""          "http://sivp.sourceforge.net/"
    "a" "SMP"               "smp"       ""
    "a" "sndfile_toolbox"   "sndfile-toolbox"   ""
    "a" "socket-server"     ""          "https://github.com/ishit/scilab-network"
    "a" "socket_toolbox"    ""          "http://www.reveyrand.fr/SOCKET2.html"
    "a" "sopi"              "sopi"      "https://github.com/pivui/sopi"
    "a" "specfun"           "specfun"   ""
    "a" "toolbox_spgl1"     "toolbox-spgl1"  ""
    "a" "spilu"             "spilu"     ""
    "a" "splspc"            "splspc"    ""
    "a" "SSP_Toolbox"       ""          "www.cdackolkata.in"
    "a" "Statpack"          "Statpack"  ""
    "a" "stftb"             "stftb"     ""
    "a" "stixbox"           "stixbox"   ""
    "a" "sudoku"            "sudoku"    ""
    "a" "supportvectormachine"  ""      ""
    "a" "swt"               "swt"       ""
    "a" "texmacs"           "texmacs"   ""
    "a" "Two-Dimensional_Transformations"   ""  "www.cdackolkata.in"
    "a" "uman"              ""          "http://sgougeon.free.fr/scilab/help/uman/scilab_en_US_help/uman.html"
    "a" "uncprb"            "uncprb"    ""
    "a" "usb_toolbox"       "usb-toolbox"   ""
    "a" "visa"              ""          ""
    "a" "Wavelab"           ""          "http://www-stat.stanford.edu/~wavelab/Wavelab_850/index_wavelab850.html"
    "a" "wgserialxcosio"    ""          "http://bioe.sourceforge.net/"
    "a" "WriteXmlExcel"     "writexmlexcel" ""
    "a" "xcos_stm32_toolbox" ""         "http://www.st.com"
    "a" "XCPL"              ""          ""
    "a" "xlreadwrite"       ""          "https://gitlab.com/mottelet/xlreadwrite"
    // "a" "xls_link"          "xls-link"  ""   // out of work
    "a" "xmllab"            "xmllab"    "http://xmllab.org"
    "a" "xmltodocbook"      "xmltodocbook"  ""
    ]
    // Are there new modules in the actual ATOMS list?
    // We get the actual list:
    tmp =  uman_get_actual_atoms_list();
    // We get ATOMS references not already registered in uman:
    // setdiff(A,B) == A entries not in B
    [tmp2, k] = setdiff(tmp(:,2), atoms(:,2))
    tmp = tmp(k,:)
    // We reject blacklisted entries
    blacklist = ["Cardiovascular"   // obsolete
                 "xls_link"         // out of work
                 "toolbox_skeleton" // no functions in it
                 "toolbox_1"
                 "toolbox_4"
                 "toolbox_5"
                 "asd"
                 "qwerraa"
                 "test"
                 "computing_experiment"
                 "indexed_names"
                 "remote_exec"
                 "generate_indexed_tasks"
                 "create_formal_model_system_dynamic_proj"
                 "formal_modeling_system_dynamic"
                 "manage_distributed_tasks"
                 "manage_dist_tasks_local_form_model_cmd"
                 "manage_distrib_tasks_formal_model_ccf"
                 "manage_distrib_tasks_formal_modeling"
                 "manage_distributed_tasks_local"
                 "manage_distrib_tasks_local_form_model"
                 "executing_task_sce"
                 "executing_task_sce_base"
                 "executing_task_sce_base_local"
                 "executing_task_sce_base_cmd"
                 "executing_task_sce_base_cmd_file"
                 "executing_task_sce_base_mpi"
                 "executing_task_sce_attributes"
                 "executing_task_sce_attributes_mpi"
                 "executing_task_sce_administrate"
                 "cmd_using_file_scilab"
                 "cmd_file_scilab"
                 "mpi_file_scilab_base"
                 "mpi_file_scilab"
                 ""
                 ]
    [tmp2, k] = setdiff(tmp(:,2), blacklist)
    // We append these references to the uman list:
    atoms = [atoms ; tmp(k,:)]

    // list on 2015-05-09
    fileExchange = [
    "fe" "addblanks"     ""  "128000"
    "fe" "bezier"        ""  "293000"
    "fe" "browseURL"     ""  "453000"
    "fe" "BVPsolver"     ""  "217000"
    "fe" "circfit"       ""  "194000"
    "fe" "combine"       ""  "237000"
    "fe" "contour3d"     ""  "contour3d"
    "fe" "cshift"        ""  "161000"
    "fe" "cummax"        ""  "348000"
    "fe" "cummin"        ""  "348000"
    "fe" "dither"        ""  "292000"
    "fe" "dxf2sci"       ""  "131000"
    "fe" "ellipfit"      ""  "194000"
    "fe" "ellipsoid"     ""  "123000"
    "fe" "ex355p"        ""  "341000"
    "fe" "fct"           ""  "213000"
    "fe" "FG_ISG500"     ""  "225000"
    "fe" "gini"          ""  "306000"
    "fe" "git"           ""  "115000"
    "fe" "GnuSciPlot"    ""  "487000"
    "fe" "ifct"          ""  "214000"
    "fe" "integratePoly" ""  "228000"
    "fe" "isDocked"      ""  "360000"
    "fe" "Ishikawa"      ""  "485000"
    "fe" "LDS1200"       ""  "226000"
    "fe" "legends_mc"    ""  "274000"
    "fe" "linfit"        ""  "250000"
    "fe" "logistics_function" "" "124000"
    "fe" "lorenzc"       ""  "307000"
    "fe" "mesharea"      ""  "370000"
    "fe" "mlbs"          ""  "mlbs"
    "fe" "mm3136A"       ""  "227000"
    "fe" "m2tabular"     ""  "297000"
    "fe" "M38XR"         ""  "232000"
    "fe" "OLPpower"      ""  "223000"
    "fe" "orderfields"   ""  "349000"
    "fe" "osciOX80"      ""  "234000"
    "fe" "partfrac"      ""  "451000"
    "fe" "pdiv_inc"      ""  "449000"
    "fe" "point_in_polygon" "" "121000"
    "fe" "polyfit"       ""  "249000"
    "fe" "polygonmetrics" "" "445000"
    "fe" "polyroots"     ""  "362000"
    "fe" "rotate3d"      ""  "369000"
    "fe" "rungekutta"    ""  "151000"
    "fe" "STL"           ""  "490000"
    "fe" "strrand"       ""  "367000"
    "fe" "strrepeat"     ""  "365000"
    "fe" "struct2json"   ""  "332000"
    "fe" "thermaSnap"    "" "230000"
    "fe" "triangu3D"     ""  "291000"
    "fe" "varspace"      ""  "364000"
    "fe" "wrscilib"      ""  "wrscilib"
    "fe" "XCPL_toolbox"  ""  ""
    "fe" "zaber"         ""  "255000"

    "fe" "polar2D"       ""  "285000"
    "fe" "polar3D"       ""  "285000"
    "fe" "triax"         ""  "285000"

    "fe" "tachometer"    ""  "312000"
    "fe" "thermometer"   "" "312000"
    "fe" "variometer"    "" "312000"
    "fe" "windrose"      ""  "312000"

    //"fe" ""    ""  ""
    //"fe" "px2rc"       ""  "335000"
    //"fe" "strsubst"    ""  "294000"
    //"fe" "rana"        ""  "172000"
    //"fe" "thermocouples" ""  "248000"
    //"fe" "isprime"     ""  "235000"
    //"fe" "peak_detect" ""  "209000"
    //"fe" "linear"      ""  "118000"
    ]
    FEroot = "https://fileexchange.scilab.org/toolboxes/"
    fileExchange(:,4) = FEroot + fileExchange(:,4)

    // Forges without ATOMS
    forges = [
    "fo" "scimax"   ""  ""
    ]

    others = [
    "o" "COMPleib"  ""  "https://github.com/pivui/scilabTools/tree/master/COMPleib"
    "o" "GPIB"      ""  "http://www.reveyrand.fr/GPIB.html"
    "o" "int4sci"   ""  "https://github.com/Int4Sci/Int4Sci-v2"
    "o" "Labview"   ""  "https://www.ni.com/fr-fr/support/documentation/supplemental/06/connecting-labview-with-scilab-enterprises-scilab-software.html"
    ];
    table = [atoms ; fileExchange ; forges ; others]
endfunction
// ==================================================================

// Copyright (C)  2013 - 2022   Samuel GOUGEON
//
// This file must be used under the terms of the CeCILL.
// Its terms are available at
// http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
//
// Utility function for uman()
function tf = uman_isATOMSchanged()
    // * Returns %T if the list of installed ATOMS modules has changed.
    //   Then the new list is recorded in SCIHOME/uman/atomsInstalledList.txt
    // * Returns %F otherwise
    aL = atomsGetInstalled();
    aL = aL(:,1:4)
    tf = %t
    filename = SCIHOME + "/uman/atomsInstalledList.txt";
    fmt = "%s\t%s\t%s\t%s\n";
    Fid = mopen(filename,"a+")
    f = mfscanf(-1, Fid, fmt)
    mclose()
    tf = size(f,1)~=size(aL,1)
    if ~tf, tf = or(f~=aL), end     // does not use || to be Scilab 5 parsable
    if tf then
        Fid = mopen(filename,"w")
        if aL~=[]
            mfprintf(Fid, fmt, aL);
        else
            mfprintf(Fid, "", []);
        end
        mclose(Fid)
    end
endfunction

// ==================================================================

// Copyright (C)  2013 - 2022   Samuel GOUGEON
//
// This file must be used under the terms of the CeCILL.
// Its terms are available at
// http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
//
// Utility function for uman()
function resu = uman_getRemoved(pattern)
    // pattern: the reference to search
    // resu:
    //  * [] if the pattern does not match any removed functions
    //  * resu = [fname replacement last_scilab_version]: vector of 3 strings
    removed = [..
//        "sound"         "playsnd"         "2023"
        "%sn"           "ellipj"        "6.1.1"
        "closeEditvar"  "close"         "6.1.1"
        "champ1"        "champ"         "6.1.1"
        "datatipToggle" "datatipManagerMode"  "6.1.1"
        "dirname"       "fileparts"     "6.1.1"
        "fplot2d"       "plot"          "6.1.1"
        "get_figure_handle" "findobj"   "6.1.1"
        "hist3d"        "bar3d"         "6.1.1"
        "noisegen"      "grand"         "6.1.1"
        "plotframe"     "plot2d"        "6.1.1"
        "ric_desc"      "riccati"       "6.1.1"
        "setPreferencesValue" "xmlSetValues" "6.1.1"
        "soundsec"      ""              "6.1.1"
        "sysdiag"      "blockdiag"        "6.1.1"
        "xdel"          "close"         "6.1.1"
        "xname"         ""              "6.1.1"
        "saveafterncommands" ""         "6.1.0"
        "denom"         ""              "6.0.2"
        "eval"          "evstr"         "6.0.2"
        "eval3d"        "ndgrid"        "6.0.2"
        "frexp"         "log2"          "6.0.2"
        "getPreferencesValue" "xmlGetValues" "6.0.2"
        "hypermat"      "matrix"        "6.0.2"
        "lstsize"       "size"          "6.0.2"
        "nanmax"        "max"           "6.0.2"
        "nanmin"        "min"           "6.0.2"
        "numer"         ""              "6.0.2"
        "square"        "figure_properties" "6.0.2"
        "strcmpi"       "strcmp"        "6.0.2"
        "with_tk"       "with_module"   "6.0.2"
        "xinfo"         "figure_properties" "6.0.2"
        "xpause"        "sleep"         "6.0.2"
        "xgetech"       "gca"           "6.0.2"
        "isequalbitwise" "assert_checkequal" "5.5.2"
        "maxfiles"      ""              "5.5.2"
        "m_circle"      "hallchart"     "5.5.2"
        "plot2d1"       "plot2d"        "5.5.2"
        "xclear"        "clf"           "5.5.2"
        "datatipSetStruct"  "datatip_properties" "5.5.2"
        "datatipGetStruct"  "datatip_properties" "5.5.2"
        "fcontour2d"    "contour2d"     "5.5.2"
        "fcontour"      "contour"       "5.5.2"
        "fac3d"       "plot3d"          "5.5.2"
        "gspec"         "spec"          "5.5.2"
        "gschur"        "schur"         "5.5.2"
        "rafiter"       "taucs_chsolve" "5.5.2"
        "numdiff"       "numderivative" "5.5.2"
        "derivative"    "numderivative" "5.5.2"
        "mvvacov"       "cov"           "5.5.2"
        "perl"          ""              "5.5.2"
        "lex_sort"      "gsort"         "5.5.2"
        "jconvMatrixMethod" "jautoTranspose"  "5.5.2"
        "havewindow"    "getscilabmode" "5.5.2"
        "user"          ""              "5.5.2"
        "xpause"        "sleep"         "5.5.2"
        //"curblockc"     "curblock"      "5.5.2"
        //"extract_help_examples"  ""     "5.5.2"
        "addf"          ""              "5.5.2"
        "subf"          ""              "5.5.2"
        "mulf"          ""              "5.5.2"
        "ldivf"         ""              "5.5.2"
        "rdivf"         ""              "5.5.2"
        "cmb_lin"       ""              "5.5.2"
        "solve"         ""              "5.5.2"
        "trianfml"      ""              "5.5.2"
        "trisolve"      ""              "5.5.2"
        "parallel_run"  ""              "5.5.2"
        "parallel_concurrency"  ""      "5.5.2"
        // "block2exp"  ""              "5.5.2"  : private
        "add_profiling" "covStart"      "5.5.2"
        "remove_profiling" "covStop"    "5.5.2"
        "reset_profiling" ""            "5.5.2"
        "profile"       ""              "5.5.2"
        "showprofile"   ""              "5.5.2"
        "plotprofile"   ""              "5.5.2"
        "setbpt"        "debug"         "5.5.2"
        "delbpt"        "debug"         "5.5.2"
        "dispbpt"       "debug"         "5.5.2"
        "comp"          "exec"          "5.5.2"
        "errcatch"      ""              "5.5.2"
        "iserror"       ""              "5.5.2"
        "str2code"      "ascii"         "5.5.2"
        "code2str"      "ascii"         "5.5.2"
        "fun2string"    "string"        "5.5.2"
        "readgateway"   ""              "5.5.2"
        "getvariablesonstack"  "who"    "5.5.2"
        "gstacksize"    ""              "5.5.2"
        "stacksize"     ""              "5.5.2"
        "macr2lst"      "macr2tree"     "5.5.2"
        "bytecode"      ""              "5.5.2"
        "bytecodewalk"  ""              "5.5.2"
        "recompilefunction" ""          "5.5.2"
        "listfunctions" ""              "5.5.2"
        "%asn"          "delip"         "5.5.1"
        "chart"         "nicholschart"  "5.5.1"
        "IsAScalar"     "isscalar"      "5.5.1"
        "jmat"          "flipdim"       "5.5.1"
        "mfft"          "fft"           "5.5.1"
        "milk_drop"     ""              "5.5.1"
        "msd"           "stdev"         "5.5.1"
        "nfreq"         "tabul"         "5.5.1"
        "pcg"           "conjgrad"      "5.5.1"
        "regress"       "reglin"        "5.5.1"
        "relocate_handle"  ""           "5.5.1"
        "st_deviation"  "stdev"         "5.5.1"
        "xmltochm"      ""              "5.5.1"
        "xsetm"         ""              "5.5.1"
        "datatipContextMenu"  ""        "5.5.0"
        "datatipEventHandler" ""        "5.5.0"
        "dft"           "fft"           "5.4.1"
        "sscanf"        "msscanf"       "5.4.1"
        "fscanf"        "mfscanf"       "5.4.1"
        "fprintf"       "mfprintf"      "5.4.1"
        "demo_message"  ""              "5.4.1"
        "demo_mdialog"  ""              "5.4.1"
        "draw"          ""              "5.4.1"
        "clear_pixmap"  ""              "5.4.1"
        "winclose"      "close"         "5.4.1"
        "datatipInitStruct"  ""         "5.4.1"
        "datatipRedraw" ""              "5.4.1"
        "getfont"       ""              "5.4.1"
        "getmark"       ""              "5.4.1"
        "getlinestyle"  ""              "5.4.1"
        "getsymbol"     ""              "5.4.1"
        "with_embedded_jre"  ""         "5.4.1"
        "fit_dat"       "datafit"       "5.4.1"
        "create_palette"  ""            "5.4.1"
        "chartooem"     ""              "5.4.0"
        "oemtochar"     ""              "5.4.0"
        "config"        "preferences"   "5.4.0"
        "createpopup"   "uicontextmenu" "5.4.0"
        "mtlb_conv"     "conv"          "5.4.0"
        "mtlb_repmat"   "repmat"        "5.4.0"
        "neldermead_display"    "disp"  "5.4.0"
        "nmplot_display"        "disp"  "5.4.0"
        "optimbase_display"     "disp"  "5.4.0"
        "optimsimplex_print"    "disp"  "5.4.0"
        "optimsimplex_tostring" "string" "5.4.0"
        "ricc_old"      "ricc"          "5.4.0"
        "showalluimenushandles"  "set(get(0),""ShowHiddenHandles"", ""on"")"  "5.4.0"
        "with_pvm"      "getversion"    "5.4.0"
        "with_texmacs"  ""              "5.4.0"
        "xbasr"         ""              "5.4.0"
        "xselect"       "show_window"   "5.4.0"
        "MSDOS"         "getos"         "5.3.3"
        "sd2sci"        ""              "5.3.3"
        "oldplot"       ""              "5.3.3"
        "readc_"        "input"         "5.2.2"
        "with_gtk"      "getversion"    "5.2.2"
        "xbasc"         "clf"           "5.2.2"
        "getf"          "exec"          "5.2.2"
        "sort"          "gsort"         "5.2.2"
        "NumTokens"     "tokens"        "5.2.1"
        "maxi"          "max"           "5.2.2"
        "mini"          "min"           "5.2.2"
        "oldbesseli"    "besseli"       "5.2.2"
        "oldbesselj"    "besselj"       "5.2.2"
        "oldbesselk"    "besselk"       "5.2.2"
        "oldbessely"    "bessely"       "5.2.2"
        "texprint"     "prettyprint"    "5.2.2"
        "pol2tex"       "prettyprint"   "5.2.2"
        "tk_savefile"   "uiputfile"     "5.2.2"
        "tk_getdir"     "uigetdir"      "5.2.2"
        "tk_choose"     "x_choose"      "5.2"
        "x_message_modeless"  "messagebox" "5.2.2"
        "sethomedirectory" "SCIHOME"    "5.2.2"
        "getcwd"        "pwd"           "5.2"
        "scilab_demos"  "demo_gui"      "5.2"
        "oldsave"       "save"          "5.1.1"
        "oldload"       "load"          "5.1.1"
        "sci2excel"     "csvWrite"      "5.1.1"
        "excel2sci"     "csvRead"       "5.1.1"
        "tk_getfile"    "uigetfile"     "5.1.1"
        "xgetfile"      "uigetfile"     "5.1.1"
        "lgfft"         ""              "5.1.1"
        "mtlb_load"     "loadmatfile"   "5.0"
        "mtlb_save"     "savematfile"   "5.0"
        "xbasimp"       "toprint|xs2ps" "5.0"
        "xg2ps"         "xs2ps"         "5.0"
        "hidetoolbar"   "toolbar"       "5.0"
        "browsehelp"    "helpbrowser"   "5.0"
        "quapro"        "qpsolve"       "5.0"
        "%sp_eye"       "speye"         "5.0"
        "TCL_gcf"       "gcf"           "5.0"
        "TCL_scf"       "scf"           "5.0"
        "TK_EvalStr"    "TCL_EvalStr"   "5.0"
        "sciGUIhelp"    "help"          "5.0"
        "demoplay"      "demo_gui"      "5.0"
        "buttondialog"  "messagebox"    "5.0"
        "tk_getvalue"   "getvalue"      "5.0"
        "xclea"         "xfrect"        "4.1.2"
        "xaxis"         "drawaxis"      "4.1.2"
        "loadplots"     ""              "4.1.2"
        "xtape"         "driver"        "4.1.2"
        "loaddefaultbrowser" ""         "4.1.2"
        "%browsehelp"   ""              "4.1.2"
        ];
    // XCOS BLOCKS
    removed = [removed
        "WFILE_f"       "WRITEC_f"      "5.5.2"
        ]
        if pattern==convstr(pattern) then
            k = find(convstr(removed(:,1))==pattern)
        else
            k = find(removed(:,1)==pattern)
        end
        resu = removed(k,:)
endfunction

// ==================================================================

// Copyright (C)  2013 - 2022   Samuel GOUGEON
//
// This file must be used under the terms of the CeCILL.
// Its terms are available at
// http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
//
// Utility function for uman()
function resu = uman_strsubst(text, pattern, replace, option)
    // option: "r" if the pattern is a regular expression (with delimiters and options)
    // replace: Group id: with TCL version = "\#"    with Java = "$#"
    Sclass = "java.lang.String"

    rhs = argn(2)

    if rhs>2 then
        if text==[] then
            resu = []
            return
        end

        sizeText = size(text)

        // Chosing the Java method: litteral, regexp, regexp once
        // ------------------------------------------------------
        if rhs>3 then
            if option=="r" then
                jmeth = 1  // "replaceAll"
            elseif or(option==["ro" "or"])
                jmeth = 3  // "replaceFirst"
            else
                msg = _("%s: Wrong value for input argument #%d: Must be in the set {%s}.\n")
                error(msprintf(msg, "strsubst", 4, "''r'',''ro''"))
            end
        else
            jmeth = 2  // "replace"
            option = ""
        end

        // Compiling the meta-method
        // -------------------------
        try
            arrayRep = jnewInstance("arrayRep")
        catch
            umanString = jimport(Sclass, %f)
            arrayRep = jcompile("arrayRep", ..
            ["public class arrayRep {"
             "public static String[] replaceInArray(int meth, String arr[], String[] pat, String[] rep) {"
             "    int nbelem = arr.length; "
             "    String tmp;"
             "    String[] out = new String[nbelem];"
             "    if (meth==1) { "
             "    for (int i = 0; i < nbelem; i++) { "
             "        tmp = arr[i]; "
             "        for (int p = 0; p < pat.length; p++) { "
             "            tmp = tmp.replaceAll(pat[p], rep[p]); "
             "            } "
             "        out[i] = tmp; "
             "        } "
             "    } "
             "    else if (meth==2) {"
             "    for (int i = 0; i < nbelem; i++) { "
             "        tmp = arr[i]; "
             "        for (int p = 0; p < pat.length; p++) { "
             "            tmp = tmp.replace(pat[p], rep[p]); "
             "            } "
             "        out[i] = tmp; "
             "        } "
             "    } "
             "    else {"
             "    for (int i = 0; i < nbelem; i++) { "
             "        tmp = arr[i]; "
             "        for (int p = 0; p < pat.length; p++) { "
             "            tmp = tmp.replaceFirst(pat[p], rep[p]); "
             "            } "
             "        out[i] = tmp; "
             "        } "
             "    } "
             "    return out; "
             "    }"
             "}"]);
         end
        //pattern = stripblanks(pattern)

        // Pre-processing the patterns: Java does not accept "/../modifiers" syntax.
        //  - modifiers must be inserted as (?#) switches in head of regexp
        //  - delimiters are not expected
        if option~="" then
            // http://docs.oracle.com/javase/tutorial/essential/regex/pattern.html
            // Extracting and checking delimiters:
            delims = part(pattern, 1)
            // Is at least one of opening delimiters a forbiden character?
            forbid = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
            if ~and(strcspn(delims, forbid)==length(delims)) then
                msg = _("uman","%s: Delimiters must be non alphanumeric.\n")
                error(msprintf(msg, "uman_strsubst"))
            end
            // Checking the closing delimiter and extracting modifiers:
            modifiers = strrchr(pattern, delims) // should include the closing delimiter
            Nunclosed = length(find((modifiers==pattern)))
            if Nunclosed>0
                msg = _("uman", "%s: the delimiter in the pattern is sometimes unclosed (''%d'' pattern(s) concerned).\n")
                error(msprintf(msg, "uman_strsubst", Nunclosed))
            end
            // Supported modifiers:
            // u => (?u) : unicode case: set by default
            // i => (?i) : case insensitive
            // m => (?m) : multiline
            // x => (?x) : ignores comments starting with # until the end of the line
            // s => (?s) : "." matches also EOL
            Modifiers = emptystr(modifiers)+"(?u"
            if or(length(modifiers)>1)
                for m = ["i" "m" "x" "s"]
                    im = grep(modifiers,m)
                    if im~=[]
                        Modifiers(im) = Modifiers(im)+m
                    end
                end
            end
            Modifiers = Modifiers + ")"
            pattern = strncpy(pattern, length(pattern)-strcspn(strrev(pattern), delims)-1)
            L = length(pattern)
            for i = 1:length(L)
                pattern(i) = Modifiers(i) + part(pattern(i), 2:L(i))
            end
        end

         // Formating argins
         // ----------------
        text    = text(:)'
        pattern = pattern(:)'
        replace = replace(:)'
        if  size(pattern,"*")==1 then
            replace = replace(1)
        elseif size(replace,"*")==1
            replace = emptystr(pattern) + replace
        elseif size(replace,"*")~=size(pattern,"*")
            msg = _("%s: Incompatible input arguments #%d and #%d: Same sizes expected.\n")
            error(msprintf(msg, "strsubst", 2, 3))
        end

        // Calling the routine
        // -------------------
        resu = arrayRep.replaceInArray(jmeth, text, pattern, replace)

        // Post-formatting the result
        resu = matrix(resu, sizeText)

    else
        resu = ""
    end
endfunction

// ==================================================================

// Copyright (C)  2013 - 2022   Samuel GOUGEON
//
// This file must be used under the terms of the CeCILL.
// Its terms are available at
// http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
//
// Utility function for uman()

function uman_settings()
    // Internal function: sets uman global configuration variables
    // according to the uman Preferences file.

    File = SCIHOME+"/uman_preferences.xml";
    if ~isfile(File)
        i = atomsGetInstalled();
        k = find(i(:,1)=="uman");
        File = i(k($), 4) + "/etc/uman_preferences.xml";
    end
    File = pathconvert(File, %f);
    doc = xmlRead(File);

    a = ["AlignSyntaxes" "DefaultCallOptions" "MaxLinesWidth", "AsciiStyling" ..
         "ListOnlyUnresolvedBugs" "SeeAlsoOnlyKeywords" "PrintHyperlinks" ..
         "AllowWideTables" ];
     if sciVer > 6.01
        v = xmlGetValues("//uman", a, doc);
     else
        v = getPreferencesValue("//uman", a, doc);
     end

    global umanAlignSyntaxes umanMaxLinesWidth umanAsciiStyling umanUserPaths
    global umanDefaultCallOptions umanListOnlyUnresolvedBugs umanSeeAlsoOnlyKeywords
    global umanPrintHyperlinks umanAllowWideTables
    umanAlignSyntaxes = v(1)
    umanNoAsciiStyling = v(4)=="checked"
    umanMaxLinesWidth = evstr(v(3))
    umanDefaultCallOptions = stripblanks(v(2))
    umanListOnlyUnresolvedBugs = v(5)=="checked"
    umanSeeAlsoOnlyKeywords = v(6)=="checked"
    umanPrintHyperlinks = v(7)
    umanAllowWideTables = v(8)

    // paths
    tmp = xmlXPath(doc, "//umanPaths"); tmp = tmp(1);
    umanUserPaths = stripblanks(tmp.content)
    xmlDelete(doc);
    if umanUserPaths=="" then
        umanUserPaths = []
    end
endfunction
// ------------------------------------------------------------------

// Copyright (C)  2013 - 2022   Samuel GOUGEON
//
// This file must be used under the terms of the CeCILL.
// Its terms are available at
// http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
//
// TCL implementation: supports Scilab < 5.5. Mac OS not supported

//function out = uman_strsubst(text, pattern, replace, option)
//    // option : unused here (for compatibility)
//   // https://gitlab.com/scilab/scilab/-/issues/12808
//    pattern0 = pattern
//    replace0 = replace
//    if size(pattern,"*")==1 then
//        pattern = "set RE {" + pattern + "}"
//        replace = "set RP {" + replace(1) + "}"
//        TCL_EvalStr(pattern)
//        TCL_EvalStr(replace)
//        out = text
//        for i = 1:size(text,"*")
//            texte = "set TXT {" + text(i) + "}"
//            try
//                TCL_EvalStr(texte)
//            catch
//                if text(i)=="\"
//                    TCL_EvalStr("set TXT {\\}")
//                else
//                    write(%io(2), lasterror())
//                    whereami()
//                    return
//                end
//            end
//            out(i) = TCL_EvalStr("regsub -all $RE $TXT $RP")
//            // http://www.tcl.tk/man/tcl8.6/TclCmd/regsub.htm
//        end
//    else
//        for i = 1:min([size(pattern,"*") size(replace,"*")])
//            text = uman_strsubst(text, pattern(i), replace(i), "r")
//        end
//        out = text
//    end
//    //t = emptystr(100,100) + html;
//    //tic(),uman_strsubst(t,"(<.*?>)","");toc()
//    //// vectorized with strcat() + processing + strsplit()   = 153 s
//    //// vectorized with a for loop                           = 38.8s
//endfunction
// ==================================================================
// CONSTRAINING EXTERNAL BUGS :
//  html parsing : https://gitlab.com/scilab/scilab/-/issues/12748 (worked around)
//  parsing "<"  : https://gitlab.com/scilab/scilab/-/issues/13877 (worked around)
//  parsing <pre>: https://gitlab.com/scilab/scilab/-/issues/12588 (worked around)
//  strsubst     : https://gitlab.com/scilab/scilab/-/issues/12808 (worked around)
//  diary        : https://gitlab.com/scilab/scilab/-/issues/12551 (ignored)
//  spaces in <code>: https://gitlab.com/scilab/scilab/-/issues/15335 (blocking)
// ----------------------------------------------------------------------------
