//
// "uman" module - http://atoms.scilab.org/toolboxes/uman
// Copyright (C) - 2015-2022 - Samuel GOUGEON
// 
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING.  
// The terms are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
//
// =================================================================================
setlanguage en;

// Page with several xml:id
uman bessel u    // Usages for all bessel# functions are displayed
uman besselh u   // Only usages for besseli are displayed

// Page without short description
uman codechecking_criteria

// Synopses
//  with white-line-separated blocks
uman rand u     // Only one block with inner blank lines
uman isequal u  // Two blocks without inner blank lines
uman grand u    // Several blocks + inner blank lines
//
uman and_op u   //  with no line including the xml:id
uman modulo     // 2 xml:id, one block each => Only modulo one displayed

// Pages using non-breakable spaces
uman bitor h
uman scatter d

// Pages with nested terms, itemized, and ordered lists 
uman brackets d

// Page with multiples Description-like sections
uman grand a

// Indented contents. Warning and note
uman dasrt p

// Pages with <latex alt> formulae
uman kron d  // multiline alt (with \n)

// Pages with examples including some literal <tags>.. </tags>
uman xmlXPath e

// Page with a <screen> area
uman ndgrid el

// Page with URLS including "&"
uman hilb cde

// See also section
// ----------------
uman uman    // * Items belonging to the uman module are displayed with their
             //   short description.
             // * Items belonging to Scilab: short description = item's label
             // * Items being URL: URL displayed (without —)
uman slint   // Missing short description for codechecking_criteria

// History section
// ---------------
uman isoview h    // Itemized (1 level): multiline without wrapping
uman bitor h      // Itemized (1 level): multiline with wrapping
uman uman h       // Itemized (1 level): multiline mixte
uman uicontrol h  // Itemized (1 level): mixte inside a revision.
uman int8 h       // Table in history
uman iconvert h   // Table in history


// Pages with Tables
// =================
uman gettext d        // <screen> in cells
uman atomsSetConfig d // Wide table  <= umanAllowWideTables
uman EXTRACTBITS e    // Wide tables <= umanAllowWideTables
uman m2sci_diag e     // programlisting in a 2-cell table
uman PlotSparse p     // 2 tables without borders
uman atomsInstall p   // tables in a itemlist, with borders & long wrapped lines
uman atomsGetInstalledPath p    // Table with a missing cell
uman codechecking_criteria d    // Nested tables
uman EXPRESSION p   // in an Xcos page
uman mtlb_int16 d   // with borders

// Tables with colspan:
uman spec d    // with borders    + Multiple colspans in headers line
uman setdiff d // without borders + Multiple colspans in headers line
uman keys      // with borders
uman typeof d  // without borders
uman empty     // 2 consecutive rows with colspan => ISSUE: NOT DISPLAYED
uman file p    //> 6.0.2

// Tables in variablelist: file, fileinfo
uman param3d p      // Tables| in nested variablelist
uman EXPRESSION     // Unnested table in an itemized list
// Other tables: isoview, colordef|, int8,  cond, calerf, consolebox

// Language switches
// -----------------
uman linspace alfr
uman linspace alru // Russian version of linspace's page
uman matrix alja   // English reference version

// Redirections / equivalences
// ---------------------------
// Operators or symbols. Internal shortcut to kron()
uman .*. cd
 
// Inner redirections to pages or summaries :
uman keys     // => console
uman colors   // => "color -s"

// Scilab equivalences of external patterns inexisting in Scilab
uman polyval    // Automatic Octave => Scilab redirection performed when possible.
                // polyval() does not exist in Scilab but is the Octave
                // function for Scilab's horner() => horner's page is displayed

// Scilab equivalence of external faux-amis			  
uman end x  // "x" targets in priority an eXternal meaning for "end". In Octave, "end" 
            // means "index of last element", as "$" in Scilab => "$" page is displayed
 
// With an ATOMS package
// ---------------------
try
    uman atoms d           // List of ATOMS functions
    yn = atomsIsInstalled("plotlib"); 
    atomsInstall plotlib ; // Let's install Plotlib
    atomsIsLoaded plotlib  // => %F
    uman plot de      // the Scilab's plot() page is targeted in priority
    uman plot dex     // "x" for the eXternal plot() page from Plotlib to be targeted
    uman plot dexg    // the help browser called with "g" does not come, because Plotlib is not loaded
    uman quiver b    // targets the ONLINE bug page of the plotlib module on the Scilab forge.
    if ~yn, atomsRemove("plotlib"), end    // cleaning after the example
catch
end
